# flake8: noqa

"""
    FactSet Fundamentals API

    Gain access to current, comprehensive, and comparative information on securities in worldwide developed and emerging markets. Composed of annual and interim/quarterly data, detailed historical financial statement content, per-share data, and calculated ratios, FactSet Fundamentals provides you with the information you need for a global investment perspective.<p>This API is rate-limited to 10 requests per second and 10 concurrent requests per user.</p>   # noqa: E501

    The version of the OpenAPI document: 2.2.0
    Contact: api@factset.com
    Generated by: https://openapi-generator.tech
"""


__version__ = "2.3.2"

# import ApiClient
from fds.sdk.FactSetFundamentals.api_client import ApiClient

# import Configuration
from fds.sdk.FactSetFundamentals.configuration import Configuration

# import exceptions
from fds.sdk.FactSetFundamentals.exceptions import OpenApiException
from fds.sdk.FactSetFundamentals.exceptions import ApiAttributeError
from fds.sdk.FactSetFundamentals.exceptions import ApiTypeError
from fds.sdk.FactSetFundamentals.exceptions import ApiValueError
from fds.sdk.FactSetFundamentals.exceptions import ApiKeyError
from fds.sdk.FactSetFundamentals.exceptions import ApiException



"""
    FactSet Fundamentals API

    Gain access to current, comprehensive, and comparative information on securities in worldwide developed and emerging markets. Composed of annual and interim/quarterly data, detailed historical financial statement content, per-share data, and calculated ratios, FactSet Fundamentals provides you with the information you need for a global investment perspective.<p>This API is rate-limited to 10 requests per second and 10 concurrent requests per user.</p>   # noqa: E501

    The version of the OpenAPI document: 2.2.0
    Contact: api@factset.com
    Generated by: https://openapi-generator.tech
"""


import re  # noqa: F401
import sys  # noqa: F401
from multiprocessing.pool import ApplyResult
import typing

from fds.sdk.FactSetFundamentals.api_client import ApiClient, Endpoint as _Endpoint
from fds.sdk.FactSetFundamentals.model_utils import (  # noqa: F401
    check_allowed_values,
    check_validations,
    date,
    datetime,
    file_type,
    none_type,
    validate_and_convert_types
)
from fds.sdk.FactSetFundamentals.exceptions import ApiException
from fds.sdk.FactSetFundamentals.model.batch_result_response import BatchResultResponse
from fds.sdk.FactSetFundamentals.model.batch_status_response import BatchStatusResponse
from fds.sdk.FactSetFundamentals.model.error_response import ErrorResponse



class GetBatchDataResponseWrapper:
    def __init__(self, status_code: int, response: object):
        """
        This constructor initializes the new GetBatchDataResponseWrapper
        to status_code, response

        Args:
            response (object): Raw response
            status_code (int): Http status code of the response
        """

        self.status_code = status_code
        self.response = response

    def get_status_code(self) -> int:
        """
        Returns: Http status code of the response
        """
        return self.status_code

    def get_response(self) -> object:
        """
        Returns: Raw Object response
        """
        return self.response

    def get_response_200(self) -> BatchResultResponse:
        """
        Raises: ApiException: Invalid response getter called.

        Returns: Request Response Object when the batch request has been completed and the response is created
        """
        if self.status_code != 200:
            raise ApiException(
                status=500,
                reason="Invalid response getter called. get_response_200 can't return a " + self.status_code + " response"
            )
        return self.response

    def get_response_202(self) -> BatchStatusResponse:
        """
        Raises: ApiException: Invalid response getter called.

        Returns: The batch Request has not finished and the result has NOT been created.
        """
        if self.status_code != 202:
            raise ApiException(
                status=500,
                reason="Invalid response getter called. get_response_202 can't return a " + self.status_code + " response"
            )
        return self.response




class BatchProcessingApi(object):
    """NOTE: This class is auto generated by OpenAPI Generator
    Ref: https://openapi-generator.tech

    Do not edit the class manually.
    """

    def __init__(self, api_client=None):
        if api_client is None:
            api_client = ApiClient()
        self.api_client = api_client

        self.get_batch_data_endpoint = _Endpoint(
            settings={
                'response_type': (
                  { 200: (BatchResultResponse,), 202: (BatchStatusResponse,), 404: (ErrorResponse,),  },
                  GetBatchDataResponseWrapper
                ),
                'auth': [
                    'FactSetApiKey',
                    'FactSetOAuth2'
                ],
                'endpoint_path': '/batch-result',
                'operation_id': 'get_batch_data',
                'http_method': 'GET',
                'servers': None,
            },
            params_map={
                'all': [
                    'id',
                ],
                'required': [
                    'id',
                ],
                'nullable': [
                ],
                'enum': [
                ],
                'validation': [
                ]
            },
            root_map={
                'validations': {
                },
                'allowed_values': {
                },
                'openapi_types': {
                    'id':
                        (str,),
                },
                'attribute_map': {
                    'id': 'id',
                },
                'location_map': {
                    'id': 'query',
                },
                'collection_format_map': {
                }
            },
            headers_map={
                'accept': [
                    'application/json'
                ],
                'content_type': [],
            },
            api_client=api_client
        )

        self.get_batch_status_endpoint = _Endpoint(
            settings={
                'response_type': (
                  { 201: (BatchStatusResponse,), 202: (BatchStatusResponse,), 404: (ErrorResponse,),  },
                  None
                ),
                'auth': [
                    'FactSetApiKey',
                    'FactSetOAuth2'
                ],
                'endpoint_path': '/batch-status',
                'operation_id': 'get_batch_status',
                'http_method': 'GET',
                'servers': None,
            },
            params_map={
                'all': [
                    'id',
                ],
                'required': [
                    'id',
                ],
                'nullable': [
                ],
                'enum': [
                ],
                'validation': [
                ]
            },
            root_map={
                'validations': {
                },
                'allowed_values': {
                },
                'openapi_types': {
                    'id':
                        (str,),
                },
                'attribute_map': {
                    'id': 'id',
                },
                'location_map': {
                    'id': 'query',
                },
                'collection_format_map': {
                }
            },
            headers_map={
                'accept': [
                    'application/json'
                ],
                'content_type': [],
            },
            api_client=api_client
        )


    @staticmethod
    def apply_kwargs_defaults(kwargs, return_http_data_only, async_req):
        kwargs["async_req"] = async_req
        kwargs["_return_http_data_only"] = return_http_data_only
        kwargs["_preload_content"] = kwargs.get("_preload_content", True)
        kwargs["_request_timeout"] = kwargs.get("_request_timeout", None)
        kwargs["_check_input_type"] = kwargs.get("_check_input_type", True)
        kwargs["_check_return_type"] = kwargs.get("_check_return_type", True)
        kwargs["_spec_property_naming"] = kwargs.get("_spec_property_naming", False)
        kwargs["_content_type"] = kwargs.get("_content_type")
        kwargs["_host_index"] = kwargs.get("_host_index")


    def get_batch_data(
        self,
        id,
        **kwargs
    ) -> GetBatchDataResponseWrapper:
        """Returns the response for a Batch Request   # noqa: E501

        Returns the response data for the underlying batch request that is specified by the id.  By default, this endpoint will return data as JSON. If you wish to receive your data in CSV format, you can edit the header to have the \"accept\" parameter as \"text/csv\" instead of \"application/json\".   # noqa: E501
        This method makes a synchronous HTTP request. Returns the http data only

        Args:
            id (str): Batch Request identifier.

        Keyword Args:
            _preload_content (bool): if False, the urllib3.HTTPResponse object
                will be returned without reading/decoding response data.
                Default is True. NOTE: if this API returns a file, it is the responsibility
                of the caller to close the file stream.
            _request_timeout (int/float/tuple): timeout setting for this request. If
                one number provided, it will be total request timeout. It can also
                be a pair (tuple) of (connection, read) timeouts.
                Default is None.
            _check_input_type (bool): specifies if type checking
                should be done one the data sent to the server.
                Default is True.
            _check_return_type (bool): specifies if type checking
                should be done one the data received from the server.
                Default is True.
            _spec_property_naming (bool): True if the variable names in the input data
                are serialized names, as specified in the OpenAPI document.
                False if the variable names in the input data
                are pythonic names, e.g. snake case (default)
            _content_type (str/None): force body content-type.
                Default is None and content-type will be predicted by allowed
                content-types and body.
            _host_index (int/None): specifies the index of the server
                that we want to use.
                Default is read from the configuration.
        Returns:
            GetBatchDataResponseWrapper
                Response Object
        """
        self.apply_kwargs_defaults(kwargs=kwargs, return_http_data_only=True, async_req=False)
        kwargs['id'] = \
            id
        return self.get_batch_data_endpoint.call_with_http_info(**kwargs)

    def get_batch_data_with_http_info(
        self,
        id,
        **kwargs
    ) -> typing.Tuple[GetBatchDataResponseWrapper, int, typing.MutableMapping]:
        """Returns the response for a Batch Request   # noqa: E501

        Returns the response data for the underlying batch request that is specified by the id.  By default, this endpoint will return data as JSON. If you wish to receive your data in CSV format, you can edit the header to have the \"accept\" parameter as \"text/csv\" instead of \"application/json\".   # noqa: E501
        This method makes a synchronous HTTP request. Returns http data, http status and headers

        Args:
            id (str): Batch Request identifier.

        Keyword Args:
            _preload_content (bool): if False, the urllib3.HTTPResponse object
                will be returned without reading/decoding response data.
                Default is True. NOTE: if this API returns a file, it is the responsibility
                of the caller to close the file stream.
            _request_timeout (int/float/tuple): timeout setting for this request. If
                one number provided, it will be total request timeout. It can also
                be a pair (tuple) of (connection, read) timeouts.
                Default is None.
            _check_input_type (bool): specifies if type checking
                should be done one the data sent to the server.
                Default is True.
            _check_return_type (bool): specifies if type checking
                should be done one the data received from the server.
                Default is True.
            _spec_property_naming (bool): True if the variable names in the input data
                are serialized names, as specified in the OpenAPI document.
                False if the variable names in the input data
                are pythonic names, e.g. snake case (default)
            _content_type (str/None): force body content-type.
                Default is None and content-type will be predicted by allowed
                content-types and body.
            _host_index (int/None): specifies the index of the server
                that we want to use.
                Default is read from the configuration.
        Returns:
            GetBatchDataResponseWrapper
                Response Object
            int
                Http Status Code
            dict
                Dictionary of the response headers
        """
        self.apply_kwargs_defaults(kwargs=kwargs, return_http_data_only=False, async_req=False)
        kwargs['id'] = \
            id
        return self.get_batch_data_endpoint.call_with_http_info(**kwargs)

    def get_batch_data_async(
        self,
        id,
        **kwargs
    ) -> "ApplyResult[GetBatchDataResponseWrapper]":
        """Returns the response for a Batch Request   # noqa: E501

        Returns the response data for the underlying batch request that is specified by the id.  By default, this endpoint will return data as JSON. If you wish to receive your data in CSV format, you can edit the header to have the \"accept\" parameter as \"text/csv\" instead of \"application/json\".   # noqa: E501
        This method makes a asynchronous HTTP request. Returns the http data, wrapped in ApplyResult

        Args:
            id (str): Batch Request identifier.

        Keyword Args:
            _preload_content (bool): if False, the urllib3.HTTPResponse object
                will be returned without reading/decoding response data.
                Default is True. NOTE: if this API returns a file, it is the responsibility
                of the caller to close the file stream.
            _request_timeout (int/float/tuple): timeout setting for this request. If
                one number provided, it will be total request timeout. It can also
                be a pair (tuple) of (connection, read) timeouts.
                Default is None.
            _check_input_type (bool): specifies if type checking
                should be done one the data sent to the server.
                Default is True.
            _check_return_type (bool): specifies if type checking
                should be done one the data received from the server.
                Default is True.
            _spec_property_naming (bool): True if the variable names in the input data
                are serialized names, as specified in the OpenAPI document.
                False if the variable names in the input data
                are pythonic names, e.g. snake case (default)
            _content_type (str/None): force body content-type.
                Default is None and content-type will be predicted by allowed
                content-types and body.
            _host_index (int/None): specifies the index of the server
                that we want to use.
                Default is read from the configuration.
        Returns:
            ApplyResult[GetBatchDataResponseWrapper]
        """
        self.apply_kwargs_defaults(kwargs=kwargs, return_http_data_only=True, async_req=True)
        kwargs['id'] = \
            id
        return self.get_batch_data_endpoint.call_with_http_info(**kwargs)

    def get_batch_data_with_http_info_async(
        self,
        id,
        **kwargs
    ) -> "ApplyResult[typing.Tuple[GetBatchDataResponseWrapper, int, typing.MutableMapping]]":
        """Returns the response for a Batch Request   # noqa: E501

        Returns the response data for the underlying batch request that is specified by the id.  By default, this endpoint will return data as JSON. If you wish to receive your data in CSV format, you can edit the header to have the \"accept\" parameter as \"text/csv\" instead of \"application/json\".   # noqa: E501
        This method makes a asynchronous HTTP request. Returns http data, http status and headers, wrapped in ApplyResult

        Args:
            id (str): Batch Request identifier.

        Keyword Args:
            _preload_content (bool): if False, the urllib3.HTTPResponse object
                will be returned without reading/decoding response data.
                Default is True. NOTE: if this API returns a file, it is the responsibility
                of the caller to close the file stream.
            _request_timeout (int/float/tuple): timeout setting for this request. If
                one number provided, it will be total request timeout. It can also
                be a pair (tuple) of (connection, read) timeouts.
                Default is None.
            _check_input_type (bool): specifies if type checking
                should be done one the data sent to the server.
                Default is True.
            _check_return_type (bool): specifies if type checking
                should be done one the data received from the server.
                Default is True.
            _spec_property_naming (bool): True if the variable names in the input data
                are serialized names, as specified in the OpenAPI document.
                False if the variable names in the input data
                are pythonic names, e.g. snake case (default)
            _content_type (str/None): force body content-type.
                Default is None and content-type will be predicted by allowed
                content-types and body.
            _host_index (int/None): specifies the index of the server
                that we want to use.
                Default is read from the configuration.
        Returns:
            ApplyResult[(GetBatchDataResponseWrapper, int, typing.Dict)]
        """
        self.apply_kwargs_defaults(kwargs=kwargs, return_http_data_only=False, async_req=True)
        kwargs['id'] = \
            id
        return self.get_batch_data_endpoint.call_with_http_info(**kwargs)


    def get_batch_status(
        self,
        id,
        **kwargs
    ) -> BatchStatusResponse:
        """Returns the status for a Batch Request   # noqa: E501

        Return the status for the underlying batch request that is specified by the id.  # noqa: E501
        This method makes a synchronous HTTP request. Returns the http data only

        Args:
            id (str): Batch Request identifier.

        Keyword Args:
            _preload_content (bool): if False, the urllib3.HTTPResponse object
                will be returned without reading/decoding response data.
                Default is True. NOTE: if this API returns a file, it is the responsibility
                of the caller to close the file stream.
            _request_timeout (int/float/tuple): timeout setting for this request. If
                one number provided, it will be total request timeout. It can also
                be a pair (tuple) of (connection, read) timeouts.
                Default is None.
            _check_input_type (bool): specifies if type checking
                should be done one the data sent to the server.
                Default is True.
            _check_return_type (bool): specifies if type checking
                should be done one the data received from the server.
                Default is True.
            _spec_property_naming (bool): True if the variable names in the input data
                are serialized names, as specified in the OpenAPI document.
                False if the variable names in the input data
                are pythonic names, e.g. snake case (default)
            _content_type (str/None): force body content-type.
                Default is None and content-type will be predicted by allowed
                content-types and body.
            _host_index (int/None): specifies the index of the server
                that we want to use.
                Default is read from the configuration.
        Returns:
            BatchStatusResponse
                Response Object
        """
        self.apply_kwargs_defaults(kwargs=kwargs, return_http_data_only=True, async_req=False)
        kwargs['id'] = \
            id
        return self.get_batch_status_endpoint.call_with_http_info(**kwargs)

    def get_batch_status_with_http_info(
        self,
        id,
        **kwargs
    ) -> typing.Tuple[BatchStatusResponse, int, typing.MutableMapping]:
        """Returns the status for a Batch Request   # noqa: E501

        Return the status for the underlying batch request that is specified by the id.  # noqa: E501
        This method makes a synchronous HTTP request. Returns http data, http status and headers

        Args:
            id (str): Batch Request identifier.

        Keyword Args:
            _preload_content (bool): if False, the urllib3.HTTPResponse object
                will be returned without reading/decoding response data.
                Default is True. NOTE: if this API returns a file, it is the responsibility
                of the caller to close the file stream.
            _request_timeout (int/float/tuple): timeout setting for this request. If
                one number provided, it will be total request timeout. It can also
                be a pair (tuple) of (connection, read) timeouts.
                Default is None.
            _check_input_type (bool): specifies if type checking
                should be done one the data sent to the server.
                Default is True.
            _check_return_type (bool): specifies if type checking
                should be done one the data received from the server.
                Default is True.
            _spec_property_naming (bool): True if the variable names in the input data
                are serialized names, as specified in the OpenAPI document.
                False if the variable names in the input data
                are pythonic names, e.g. snake case (default)
            _content_type (str/None): force body content-type.
                Default is None and content-type will be predicted by allowed
                content-types and body.
            _host_index (int/None): specifies the index of the server
                that we want to use.
                Default is read from the configuration.
        Returns:
            BatchStatusResponse
                Response Object
            int
                Http Status Code
            dict
                Dictionary of the response headers
        """
        self.apply_kwargs_defaults(kwargs=kwargs, return_http_data_only=False, async_req=False)
        kwargs['id'] = \
            id
        return self.get_batch_status_endpoint.call_with_http_info(**kwargs)

    def get_batch_status_async(
        self,
        id,
        **kwargs
    ) -> "ApplyResult[BatchStatusResponse]":
        """Returns the status for a Batch Request   # noqa: E501

        Return the status for the underlying batch request that is specified by the id.  # noqa: E501
        This method makes a asynchronous HTTP request. Returns the http data, wrapped in ApplyResult

        Args:
            id (str): Batch Request identifier.

        Keyword Args:
            _preload_content (bool): if False, the urllib3.HTTPResponse object
                will be returned without reading/decoding response data.
                Default is True. NOTE: if this API returns a file, it is the responsibility
                of the caller to close the file stream.
            _request_timeout (int/float/tuple): timeout setting for this request. If
                one number provided, it will be total request timeout. It can also
                be a pair (tuple) of (connection, read) timeouts.
                Default is None.
            _check_input_type (bool): specifies if type checking
                should be done one the data sent to the server.
                Default is True.
            _check_return_type (bool): specifies if type checking
                should be done one the data received from the server.
                Default is True.
            _spec_property_naming (bool): True if the variable names in the input data
                are serialized names, as specified in the OpenAPI document.
                False if the variable names in the input data
                are pythonic names, e.g. snake case (default)
            _content_type (str/None): force body content-type.
                Default is None and content-type will be predicted by allowed
                content-types and body.
            _host_index (int/None): specifies the index of the server
                that we want to use.
                Default is read from the configuration.
        Returns:
            ApplyResult[BatchStatusResponse]
        """
        self.apply_kwargs_defaults(kwargs=kwargs, return_http_data_only=True, async_req=True)
        kwargs['id'] = \
            id
        return self.get_batch_status_endpoint.call_with_http_info(**kwargs)

    def get_batch_status_with_http_info_async(
        self,
        id,
        **kwargs
    ) -> "ApplyResult[typing.Tuple[BatchStatusResponse, int, typing.MutableMapping]]":
        """Returns the status for a Batch Request   # noqa: E501

        Return the status for the underlying batch request that is specified by the id.  # noqa: E501
        This method makes a asynchronous HTTP request. Returns http data, http status and headers, wrapped in ApplyResult

        Args:
            id (str): Batch Request identifier.

        Keyword Args:
            _preload_content (bool): if False, the urllib3.HTTPResponse object
                will be returned without reading/decoding response data.
                Default is True. NOTE: if this API returns a file, it is the responsibility
                of the caller to close the file stream.
            _request_timeout (int/float/tuple): timeout setting for this request. If
                one number provided, it will be total request timeout. It can also
                be a pair (tuple) of (connection, read) timeouts.
                Default is None.
            _check_input_type (bool): specifies if type checking
                should be done one the data sent to the server.
                Default is True.
            _check_return_type (bool): specifies if type checking
                should be done one the data received from the server.
                Default is True.
            _spec_property_naming (bool): True if the variable names in the input data
                are serialized names, as specified in the OpenAPI document.
                False if the variable names in the input data
                are pythonic names, e.g. snake case (default)
            _content_type (str/None): force body content-type.
                Default is None and content-type will be predicted by allowed
                content-types and body.
            _host_index (int/None): specifies the index of the server
                that we want to use.
                Default is read from the configuration.
        Returns:
            ApplyResult[(BatchStatusResponse, int, typing.Dict)]
        """
        self.apply_kwargs_defaults(kwargs=kwargs, return_http_data_only=False, async_req=True)
        kwargs['id'] = \
            id
        return self.get_batch_status_endpoint.call_with_http_info(**kwargs)





# flake8: noqa

# import all models into this package
# if you have many models here with many references from one model to another this may
# raise a RecursionError
# to avoid this, import only the models that you directly need like:
# from from fds.sdk.FactSetFundamentals.model.pet import Pet
# or import this package, but before doing it, use:
# import sys
# sys.setrecursionlimit(n)

from fds.sdk.FactSetFundamentals.model.address import Address
from fds.sdk.FactSetFundamentals.model.batch import Batch
from fds.sdk.FactSetFundamentals.model.batch_error_object import BatchErrorObject
from fds.sdk.FactSetFundamentals.model.batch_result import BatchResult
from fds.sdk.FactSetFundamentals.model.batch_result_response import BatchResultResponse
from fds.sdk.FactSetFundamentals.model.batch_status import BatchStatus
from fds.sdk.FactSetFundamentals.model.batch_status_response import BatchStatusResponse
from fds.sdk.FactSetFundamentals.model.company_fundamentals_response import CompanyFundamentalsResponse
from fds.sdk.FactSetFundamentals.model.company_report_error_object import CompanyReportErrorObject
from fds.sdk.FactSetFundamentals.model.dividend import Dividend
from fds.sdk.FactSetFundamentals.model.error_object import ErrorObject
from fds.sdk.FactSetFundamentals.model.error_object_links import ErrorObjectLinks
from fds.sdk.FactSetFundamentals.model.error_response import ErrorResponse
from fds.sdk.FactSetFundamentals.model.exchange import Exchange
from fds.sdk.FactSetFundamentals.model.financial_response import FinancialResponse
from fds.sdk.FactSetFundamentals.model.financials import Financials
from fds.sdk.FactSetFundamentals.model.fiscal_period import FiscalPeriod
from fds.sdk.FactSetFundamentals.model.fundamental import Fundamental
from fds.sdk.FactSetFundamentals.model.fundamental_fiscal_period import FundamentalFiscalPeriod
from fds.sdk.FactSetFundamentals.model.fundamental_request_body import FundamentalRequestBody
from fds.sdk.FactSetFundamentals.model.fundamental_value import FundamentalValue
from fds.sdk.FactSetFundamentals.model.fundamentals import Fundamentals
from fds.sdk.FactSetFundamentals.model.fundamentals_request import FundamentalsRequest
from fds.sdk.FactSetFundamentals.model.fundamentals_response import FundamentalsResponse
from fds.sdk.FactSetFundamentals.model.ids_batch_max30000 import IdsBatchMax30000
from fds.sdk.FactSetFundamentals.model.item import Item
from fds.sdk.FactSetFundamentals.model.metric import Metric
from fds.sdk.FactSetFundamentals.model.metrics import Metrics
from fds.sdk.FactSetFundamentals.model.metrics_response import MetricsResponse
from fds.sdk.FactSetFundamentals.model.periodicity import Periodicity
from fds.sdk.FactSetFundamentals.model.profile import Profile
from fds.sdk.FactSetFundamentals.model.profile_response import ProfileResponse
from fds.sdk.FactSetFundamentals.model.segment import Segment
from fds.sdk.FactSetFundamentals.model.segment_request_body import SegmentRequestBody
from fds.sdk.FactSetFundamentals.model.segment_type import SegmentType
from fds.sdk.FactSetFundamentals.model.segment_value import SegmentValue
from fds.sdk.FactSetFundamentals.model.segments_periodicity import SegmentsPeriodicity
from fds.sdk.FactSetFundamentals.model.segments_request import SegmentsRequest
from fds.sdk.FactSetFundamentals.model.segments_response import SegmentsResponse
from fds.sdk.FactSetFundamentals.model.update_type import UpdateType



"""
    FactSet Fundamentals API

    Gain access to current, comprehensive, and comparative information on securities in worldwide developed and emerging markets. Composed of annual and interim/quarterly data, detailed historical financial statement content, per-share data, and calculated ratios, FactSet Fundamentals provides you with the information you need for a global investment perspective.<p>This API is rate-limited to 10 requests per second and 10 concurrent requests per user.</p>   # noqa: E501

    The version of the OpenAPI document: 2.2.0
    Contact: api@factset.com
    Generated by: https://openapi-generator.tech
"""


import re  # noqa: F401
import sys  # noqa: F401
from multiprocessing.pool import ApplyResult
import typing

from fds.sdk.FactSetFundamentals.api_client import ApiClient, Endpoint as _Endpoint
from fds.sdk.FactSetFundamentals.model_utils import (  # noqa: F401
    check_allowed_values,
    check_validations,
    date,
    datetime,
    file_type,
    none_type,
    validate_and_convert_types
)
from fds.sdk.FactSetFundamentals.exceptions import ApiException
from fds.sdk.FactSetFundamentals.model.batch_status_response import BatchStatusResponse
from fds.sdk.FactSetFundamentals.model.error_response import ErrorResponse
from fds.sdk.FactSetFundamentals.model.segments_request import SegmentsRequest
from fds.sdk.FactSetFundamentals.model.segments_response import SegmentsResponse



class GetFdsSegmentsForListResponseWrapper:
    def __init__(self, status_code: int, response: object):
        """
        This constructor initializes the new GetFdsSegmentsForListResponseWrapper
        to status_code, response

        Args:
            response (object): Raw response
            status_code (int): Http status code of the response
        """

        self.status_code = status_code
        self.response = response

    def get_status_code(self) -> int:
        """
        Returns: Http status code of the response
        """
        return self.status_code

    def get_response(self) -> object:
        """
        Returns: Raw Object response
        """
        return self.response

    def get_response_200(self) -> SegmentsResponse:
        """
        Raises: ApiException: Invalid response getter called.

        Returns: Segment data items
        """
        if self.status_code != 200:
            raise ApiException(
                status=500,
                reason="Invalid response getter called. get_response_200 can't return a " + self.status_code + " response"
            )
        return self.response

    def get_response_202(self) -> BatchStatusResponse:
        """
        Raises: ApiException: Invalid response getter called.

        Returns: Batch request has been accepted.
        """
        if self.status_code != 202:
            raise ApiException(
                status=500,
                reason="Invalid response getter called. get_response_202 can't return a " + self.status_code + " response"
            )
        return self.response




class SegmentsApi(object):
    """NOTE: This class is auto generated by OpenAPI Generator
    Ref: https://openapi-generator.tech

    Do not edit the class manually.
    """

    def __init__(self, api_client=None):
        if api_client is None:
            api_client = ApiClient()
        self.api_client = api_client

        self.get_fds_segments_for_list_endpoint = _Endpoint(
            settings={
                'response_type': (
                  { 200: (SegmentsResponse,), 202: (BatchStatusResponse,), 400: (ErrorResponse,), 401: (ErrorResponse,), 403: (ErrorResponse,), 415: (ErrorResponse,), 500: (ErrorResponse,),  },
                  GetFdsSegmentsForListResponseWrapper
                ),
                'auth': [
                    'FactSetApiKey',
                    'FactSetOAuth2'
                ],
                'endpoint_path': '/segments',
                'operation_id': 'get_fds_segments_for_list',
                'http_method': 'POST',
                'servers': None,
            },
            params_map={
                'all': [
                    'segments_request',
                ],
                'required': [
                    'segments_request',
                ],
                'nullable': [
                ],
                'enum': [
                ],
                'validation': [
                ]
            },
            root_map={
                'validations': {
                },
                'allowed_values': {
                },
                'openapi_types': {
                    'segments_request':
                        (SegmentsRequest,),
                },
                'attribute_map': {
                },
                'location_map': {
                    'segments_request': 'body',
                },
                'collection_format_map': {
                }
            },
            headers_map={
                'accept': [
                    'application/json'
                ],
                'content_type': [
                    'application/json'
                ]
            },
            api_client=api_client
        )


    @staticmethod
    def apply_kwargs_defaults(kwargs, return_http_data_only, async_req):
        kwargs["async_req"] = async_req
        kwargs["_return_http_data_only"] = return_http_data_only
        kwargs["_preload_content"] = kwargs.get("_preload_content", True)
        kwargs["_request_timeout"] = kwargs.get("_request_timeout", None)
        kwargs["_check_input_type"] = kwargs.get("_check_input_type", True)
        kwargs["_check_return_type"] = kwargs.get("_check_return_type", True)
        kwargs["_spec_property_naming"] = kwargs.get("_spec_property_naming", False)
        kwargs["_content_type"] = kwargs.get("_content_type")
        kwargs["_host_index"] = kwargs.get("_host_index")


    def get_fds_segments_for_list(
        self,
        segments_request,
        **kwargs
    ) -> GetFdsSegmentsForListResponseWrapper:
        """Returns Company Segment Data.  # noqa: E501

        Retrieves Sales Metrics data for specified companies.  The ```/segments``` endpoint currently supports Long Running asynchronous requests up to **20 minutes** via batch parameter. Id limits are increased to 30000 ids per request when using batch capability.This 30000 id limit has been derived based on single metric for one day. This feature is available for all users.   # noqa: E501
        This method makes a synchronous HTTP request. Returns the http data only

        Args:
            segments_request (SegmentsRequest): Request object for requesting segments data

        Keyword Args:
            _preload_content (bool): if False, the urllib3.HTTPResponse object
                will be returned without reading/decoding response data.
                Default is True. NOTE: if this API returns a file, it is the responsibility
                of the caller to close the file stream.
            _request_timeout (int/float/tuple): timeout setting for this request. If
                one number provided, it will be total request timeout. It can also
                be a pair (tuple) of (connection, read) timeouts.
                Default is None.
            _check_input_type (bool): specifies if type checking
                should be done one the data sent to the server.
                Default is True.
            _check_return_type (bool): specifies if type checking
                should be done one the data received from the server.
                Default is True.
            _spec_property_naming (bool): True if the variable names in the input data
                are serialized names, as specified in the OpenAPI document.
                False if the variable names in the input data
                are pythonic names, e.g. snake case (default)
            _content_type (str/None): force body content-type.
                Default is None and content-type will be predicted by allowed
                content-types and body.
            _host_index (int/None): specifies the index of the server
                that we want to use.
                Default is read from the configuration.
        Returns:
            GetFdsSegmentsForListResponseWrapper
                Response Object
        """
        self.apply_kwargs_defaults(kwargs=kwargs, return_http_data_only=True, async_req=False)
        kwargs['segments_request'] = \
            segments_request
        return self.get_fds_segments_for_list_endpoint.call_with_http_info(**kwargs)

    def get_fds_segments_for_list_with_http_info(
        self,
        segments_request,
        **kwargs
    ) -> typing.Tuple[GetFdsSegmentsForListResponseWrapper, int, typing.MutableMapping]:
        """Returns Company Segment Data.  # noqa: E501

        Retrieves Sales Metrics data for specified companies.  The ```/segments``` endpoint currently supports Long Running asynchronous requests up to **20 minutes** via batch parameter. Id limits are increased to 30000 ids per request when using batch capability.This 30000 id limit has been derived based on single metric for one day. This feature is available for all users.   # noqa: E501
        This method makes a synchronous HTTP request. Returns http data, http status and headers

        Args:
            segments_request (SegmentsRequest): Request object for requesting segments data

        Keyword Args:
            _preload_content (bool): if False, the urllib3.HTTPResponse object
                will be returned without reading/decoding response data.
                Default is True. NOTE: if this API returns a file, it is the responsibility
                of the caller to close the file stream.
            _request_timeout (int/float/tuple): timeout setting for this request. If
                one number provided, it will be total request timeout. It can also
                be a pair (tuple) of (connection, read) timeouts.
                Default is None.
            _check_input_type (bool): specifies if type checking
                should be done one the data sent to the server.
                Default is True.
            _check_return_type (bool): specifies if type checking
                should be done one the data received from the server.
                Default is True.
            _spec_property_naming (bool): True if the variable names in the input data
                are serialized names, as specified in the OpenAPI document.
                False if the variable names in the input data
                are pythonic names, e.g. snake case (default)
            _content_type (str/None): force body content-type.
                Default is None and content-type will be predicted by allowed
                content-types and body.
            _host_index (int/None): specifies the index of the server
                that we want to use.
                Default is read from the configuration.
        Returns:
            GetFdsSegmentsForListResponseWrapper
                Response Object
            int
                Http Status Code
            dict
                Dictionary of the response headers
        """
        self.apply_kwargs_defaults(kwargs=kwargs, return_http_data_only=False, async_req=False)
        kwargs['segments_request'] = \
            segments_request
        return self.get_fds_segments_for_list_endpoint.call_with_http_info(**kwargs)

    def get_fds_segments_for_list_async(
        self,
        segments_request,
        **kwargs
    ) -> "ApplyResult[GetFdsSegmentsForListResponseWrapper]":
        """Returns Company Segment Data.  # noqa: E501

        Retrieves Sales Metrics data for specified companies.  The ```/segments``` endpoint currently supports Long Running asynchronous requests up to **20 minutes** via batch parameter. Id limits are increased to 30000 ids per request when using batch capability.This 30000 id limit has been derived based on single metric for one day. This feature is available for all users.   # noqa: E501
        This method makes a asynchronous HTTP request. Returns the http data, wrapped in ApplyResult

        Args:
            segments_request (SegmentsRequest): Request object for requesting segments data

        Keyword Args:
            _preload_content (bool): if False, the urllib3.HTTPResponse object
                will be returned without reading/decoding response data.
                Default is True. NOTE: if this API returns a file, it is the responsibility
                of the caller to close the file stream.
            _request_timeout (int/float/tuple): timeout setting for this request. If
                one number provided, it will be total request timeout. It can also
                be a pair (tuple) of (connection, read) timeouts.
                Default is None.
            _check_input_type (bool): specifies if type checking
                should be done one the data sent to the server.
                Default is True.
            _check_return_type (bool): specifies if type checking
                should be done one the data received from the server.
                Default is True.
            _spec_property_naming (bool): True if the variable names in the input data
                are serialized names, as specified in the OpenAPI document.
                False if the variable names in the input data
                are pythonic names, e.g. snake case (default)
            _content_type (str/None): force body content-type.
                Default is None and content-type will be predicted by allowed
                content-types and body.
            _host_index (int/None): specifies the index of the server
                that we want to use.
                Default is read from the configuration.
        Returns:
            ApplyResult[GetFdsSegmentsForListResponseWrapper]
        """
        self.apply_kwargs_defaults(kwargs=kwargs, return_http_data_only=True, async_req=True)
        kwargs['segments_request'] = \
            segments_request
        return self.get_fds_segments_for_list_endpoint.call_with_http_info(**kwargs)

    def get_fds_segments_for_list_with_http_info_async(
        self,
        segments_request,
        **kwargs
    ) -> "ApplyResult[typing.Tuple[GetFdsSegmentsForListResponseWrapper, int, typing.MutableMapping]]":
        """Returns Company Segment Data.  # noqa: E501

        Retrieves Sales Metrics data for specified companies.  The ```/segments``` endpoint currently supports Long Running asynchronous requests up to **20 minutes** via batch parameter. Id limits are increased to 30000 ids per request when using batch capability.This 30000 id limit has been derived based on single metric for one day. This feature is available for all users.   # noqa: E501
        This method makes a asynchronous HTTP request. Returns http data, http status and headers, wrapped in ApplyResult

        Args:
            segments_request (SegmentsRequest): Request object for requesting segments data

        Keyword Args:
            _preload_content (bool): if False, the urllib3.HTTPResponse object
                will be returned without reading/decoding response data.
                Default is True. NOTE: if this API returns a file, it is the responsibility
                of the caller to close the file stream.
            _request_timeout (int/float/tuple): timeout setting for this request. If
                one number provided, it will be total request timeout. It can also
                be a pair (tuple) of (connection, read) timeouts.
                Default is None.
            _check_input_type (bool): specifies if type checking
                should be done one the data sent to the server.
                Default is True.
            _check_return_type (bool): specifies if type checking
                should be done one the data received from the server.
                Default is True.
            _spec_property_naming (bool): True if the variable names in the input data
                are serialized names, as specified in the OpenAPI document.
                False if the variable names in the input data
                are pythonic names, e.g. snake case (default)
            _content_type (str/None): force body content-type.
                Default is None and content-type will be predicted by allowed
                content-types and body.
            _host_index (int/None): specifies the index of the server
                that we want to use.
                Default is read from the configuration.
        Returns:
            ApplyResult[(GetFdsSegmentsForListResponseWrapper, int, typing.Dict)]
        """
        self.apply_kwargs_defaults(kwargs=kwargs, return_http_data_only=False, async_req=True)
        kwargs['segments_request'] = \
            segments_request
        return self.get_fds_segments_for_list_endpoint.call_with_http_info(**kwargs)





"""
    FactSet Fundamentals API

    Gain access to current, comprehensive, and comparative information on securities in worldwide developed and emerging markets. Composed of annual and interim/quarterly data, detailed historical financial statement content, per-share data, and calculated ratios, FactSet Fundamentals provides you with the information you need for a global investment perspective.<p>This API is rate-limited to 10 requests per second and 10 concurrent requests per user.</p>   # noqa: E501

    The version of the OpenAPI document: 2.2.0
    Contact: api@factset.com
    Generated by: https://openapi-generator.tech
"""


import re  # noqa: F401
import sys  # noqa: F401

from fds.sdk.FactSetFundamentals.model_utils import (  # noqa: F401
    ApiTypeError,
    ModelComposed,
    ModelNormal,
    ModelSimple,
    cached_property,
    change_keys_js_to_python,
    convert_js_args_to_python_args,
    date,
    datetime,
    file_type,
    none_type,
    validate_get_composed_info,
    OpenApiModel
)
from fds.sdk.FactSetFundamentals.exceptions import ApiAttributeError


def lazy_import():
    from fds.sdk.FactSetFundamentals.model.company_report_error_object import CompanyReportErrorObject
    from fds.sdk.FactSetFundamentals.model.dividend import Dividend
    globals()['CompanyReportErrorObject'] = CompanyReportErrorObject
    globals()['Dividend'] = Dividend


class Fundamentals(ModelNormal):
    """NOTE: This class is auto generated by OpenAPI Generator.
    Ref: https://openapi-generator.tech

    Do not edit the class manually.

    Attributes:
      allowed_values (dict): The key is the tuple path to the attribute
          and the for var_name this is (var_name,). The value is a dict
          with a capitalized key describing the allowed value and an allowed
          value. These dicts store the allowed enum values.
      attribute_map (dict): The key is attribute name
          and the value is json key in definition.
      discriminator_value_class_map (dict): A dict to go from the discriminator
          variable value to the discriminator class name.
      validations (dict): The key is the tuple path to the attribute
          and the for var_name this is (var_name,). The value is a dict
          that stores validations for max_length, min_length, max_items,
          min_items, exclusive_maximum, inclusive_maximum, exclusive_minimum,
          inclusive_minimum, and regex.
      additional_properties_type (tuple): A tuple of classes accepted
          as additional properties values.
    """

    allowed_values = {
    }

    validations = {
    }

    @cached_property
    def additional_properties_type():
        """
        This must be a method because a model may have properties that are
        of type self, this must run after the class is loaded
        """
        lazy_import()
        return (bool, date, datetime, dict, float, int, list, str, none_type,)  # noqa: E501

    _nullable = False

    @cached_property
    def openapi_types():
        """
        This must be a method because a model may have properties that are
        of type self, this must run after the class is loaded

        Returns
            openapi_types (dict): The key is attribute name
                and the value is attribute type.
        """
        lazy_import()
        return {
            'request_id': (str,),  # noqa: E501
            'error': (CompanyReportErrorObject,),  # noqa: E501
            'fsym_id': (str, none_type,),  # noqa: E501
            'as_of_date': (date, none_type,),  # noqa: E501
            'currency': (str, none_type,),  # noqa: E501
            'asset_turnover_ratio': (float, none_type,),  # noqa: E501
            'book_value_per_share': (float, none_type,),  # noqa: E501
            'cash_flow_per_share': (float, none_type,),  # noqa: E501
            'cash_per_share': (float, none_type,),  # noqa: E501
            'current_ratio': (float, none_type,),  # noqa: E501
            'dividend': (Dividend,),  # noqa: E501
            'earnings_per_share': (float, none_type,),  # noqa: E501
            'ebit_margin': (float, none_type,),  # noqa: E501
            'ebitda_margin': (float, none_type,),  # noqa: E501
            'enterprise_value': (float, none_type,),  # noqa: E501
            'five_year_average_yield': (float, none_type,),  # noqa: E501
            'five_year_dividend_growth_rate': (float, none_type,),  # noqa: E501
            'floating_shares_outstanding': (float, none_type,),  # noqa: E501
            'gross_margin': (float, none_type,),  # noqa: E501
            'inventory_turnover': (float, none_type,),  # noqa: E501
            'long_term_debt_to_equity': (float, none_type,),  # noqa: E501
            'net_income': (float, none_type,),  # noqa: E501
            'number_of_employees': (int, none_type,),  # noqa: E501
            'payout_ratio': (float, none_type,),  # noqa: E501
            'pretax_margin': (float, none_type,),  # noqa: E501
            'price_to_book_ratio': (float, none_type,),  # noqa: E501
            'price_to_cash_flow_ratio': (float, none_type,),  # noqa: E501
            'price_to_earnings_ratio': (float, none_type,),  # noqa: E501
            'price_to_sales_ratio': (float, none_type,),  # noqa: E501
            'quick_ratio': (float, none_type,),  # noqa: E501
            'return_on_assets': (float, none_type,),  # noqa: E501
            'return_on_equity': (float, none_type,),  # noqa: E501
            'return_on_invested_capital': (float, none_type,),  # noqa: E501
            'revenue_per_share': (float, none_type,),  # noqa: E501
            'sales_per_employee': (float, none_type,),  # noqa: E501
            'sales_per_share': (float, none_type,),  # noqa: E501
            'share_holder_equity': (float, none_type,),  # noqa: E501
            'three_year_average_yield': (float, none_type,),  # noqa: E501
            'three_year_dividend_growth_rate': (float, none_type,),  # noqa: E501
            'total_assets': (float, none_type,),  # noqa: E501
            'total_debt_to_equity': (float, none_type,),  # noqa: E501
            'total_revenue': (float, none_type,),  # noqa: E501
            'trailing_twelve_month_earnings_per_share': (float, none_type,),  # noqa: E501
        }

    @cached_property
    def discriminator():
        return None


    attribute_map = {
        'request_id': 'requestId',  # noqa: E501
        'error': 'error',  # noqa: E501
        'fsym_id': 'fsymId',  # noqa: E501
        'as_of_date': 'asOfDate',  # noqa: E501
        'currency': 'currency',  # noqa: E501
        'asset_turnover_ratio': 'assetTurnoverRatio',  # noqa: E501
        'book_value_per_share': 'bookValuePerShare',  # noqa: E501
        'cash_flow_per_share': 'cashFlowPerShare',  # noqa: E501
        'cash_per_share': 'cashPerShare',  # noqa: E501
        'current_ratio': 'currentRatio',  # noqa: E501
        'dividend': 'dividend',  # noqa: E501
        'earnings_per_share': 'earningsPerShare',  # noqa: E501
        'ebit_margin': 'ebitMargin',  # noqa: E501
        'ebitda_margin': 'ebitdaMargin',  # noqa: E501
        'enterprise_value': 'enterpriseValue',  # noqa: E501
        'five_year_average_yield': 'fiveYearAverageYield',  # noqa: E501
        'five_year_dividend_growth_rate': 'fiveYearDividendGrowthRate',  # noqa: E501
        'floating_shares_outstanding': 'floatingSharesOutstanding',  # noqa: E501
        'gross_margin': 'grossMargin',  # noqa: E501
        'inventory_turnover': 'inventoryTurnover',  # noqa: E501
        'long_term_debt_to_equity': 'longTermDebtToEquity',  # noqa: E501
        'net_income': 'netIncome',  # noqa: E501
        'number_of_employees': 'numberOfEmployees',  # noqa: E501
        'payout_ratio': 'payoutRatio',  # noqa: E501
        'pretax_margin': 'pretaxMargin',  # noqa: E501
        'price_to_book_ratio': 'priceToBookRatio',  # noqa: E501
        'price_to_cash_flow_ratio': 'priceToCashFlowRatio',  # noqa: E501
        'price_to_earnings_ratio': 'priceToEarningsRatio',  # noqa: E501
        'price_to_sales_ratio': 'priceToSalesRatio',  # noqa: E501
        'quick_ratio': 'quickRatio',  # noqa: E501
        'return_on_assets': 'returnOnAssets',  # noqa: E501
        'return_on_equity': 'returnOnEquity',  # noqa: E501
        'return_on_invested_capital': 'returnOnInvestedCapital',  # noqa: E501
        'revenue_per_share': 'revenuePerShare',  # noqa: E501
        'sales_per_employee': 'salesPerEmployee',  # noqa: E501
        'sales_per_share': 'salesPerShare',  # noqa: E501
        'share_holder_equity': 'shareHolderEquity',  # noqa: E501
        'three_year_average_yield': 'threeYearAverageYield',  # noqa: E501
        'three_year_dividend_growth_rate': 'threeYearDividendGrowthRate',  # noqa: E501
        'total_assets': 'totalAssets',  # noqa: E501
        'total_debt_to_equity': 'totalDebtToEquity',  # noqa: E501
        'total_revenue': 'totalRevenue',  # noqa: E501
        'trailing_twelve_month_earnings_per_share': 'trailingTwelveMonthEarningsPerShare',  # noqa: E501
    }

    read_only_vars = {
    }

    _composed_schemas = {}

    @classmethod
    @convert_js_args_to_python_args
    def _from_openapi_data(cls, *args, **kwargs):  # noqa: E501
        """Fundamentals - a model defined in OpenAPI

        Keyword Args:
            _check_type (bool): if True, values for parameters in openapi_types
                                will be type checked and a TypeError will be
                                raised if the wrong type is input.
                                Defaults to True
            _path_to_item (tuple/list): This is a list of keys or values to
                                drill down to the model in received_data
                                when deserializing a response
            _spec_property_naming (bool): True if the variable names in the input data
                                are serialized names, as specified in the OpenAPI document.
                                False if the variable names in the input data
                                are pythonic names, e.g. snake case (default)
            _configuration (Configuration): the instance to use when
                                deserializing a file_type parameter.
                                If passed, type conversion is attempted
                                If omitted no type conversion is done.
            _visited_composed_classes (tuple): This stores a tuple of
                                classes that we have traveled through so that
                                if we see that class again we will not use its
                                discriminator again.
                                When traveling through a discriminator, the
                                composed schema that is
                                is traveled through is added to this set.
                                For example if Animal has a discriminator
                                petType and we pass in "Dog", and the class Dog
                                allOf includes Animal, we move through Animal
                                once using the discriminator, and pick Dog.
                                Then in Dog, we will make an instance of the
                                Animal class but this time we won't travel
                                through its discriminator because we passed in
                                _visited_composed_classes = (Animal,)
            request_id (str): Identifier that was used for the request.. [optional]  # noqa: E501
            error (CompanyReportErrorObject): [optional]  # noqa: E501
            fsym_id (str, none_type): FactSet Regional Security Identifier. Six alpha-numeric characters, excluding vowels, with an -R suffix (XXXXXX-R). Identifies the security's best regional security data series per currency. For equities, all primary listings per region and currency are allocated a regional-level permanent identifier. The regional-level permanent identifier will be available once a SEDOL representing the region/currency has been allocated and the identifiers are on FactSet.. [optional]  # noqa: E501
            as_of_date (date, none_type): Date on which the specified fundamentals data or information is accurate or relevant.. [optional]  # noqa: E501
            currency (str, none_type): Currency code for the data. For a list of currency ISO codes, visit [Online Assistant Page #1470](https://oa.apps.factset.com/pages/1470).. [optional]  # noqa: E501
            asset_turnover_ratio (float, none_type): The asset turnover ratio measures the value of a company's sales or revenues relative to the value of its assets. [optional]  # noqa: E501
            book_value_per_share (float, none_type): Book value per common share is a formula used to calculate the per share value of a company based on common shareholders' equity in the company. [optional]  # noqa: E501
            cash_flow_per_share (float, none_type): Cash flow per share is calculated as a ratio, indicating the amount of cash a business generates based on a company's net income with the costs of depreciation and amortization added back. [optional]  # noqa: E501
            cash_per_share (float, none_type): Cash Per Share of Security. [optional]  # noqa: E501
            current_ratio (float, none_type): The current ratio is a liquidity ratio that measures a company's ability to pay short-term and long-term obligations. The ratio is calculated by comparing current assets to current liabilities. [optional]  # noqa: E501
            dividend (Dividend): [optional]  # noqa: E501
            earnings_per_share (float, none_type): Earnings per share (EPS) is the portion of a company's profit allocated to each share of common stock. [optional]  # noqa: E501
            ebit_margin (float, none_type): EBIT margin is an assessment of a firm's operating profitability as a percentage of its total revenue. It is equal to earnings before interest and tax (EBIT) divided by total revenue. [optional]  # noqa: E501
            ebitda_margin (float, none_type): EBITDA margin is an assessment of a firm's operating profitability as a percentage of its total revenue. It is equal to earnings before interest, tax, depreciation and amortization (EBITDA) divided by total revenue. [optional]  # noqa: E501
            enterprise_value (float, none_type): Enterprise Value (EV) is the measure of a company's total value for the period and date(s) requested in local currency by default. [optional]  # noqa: E501
            five_year_average_yield (float, none_type): Average of the dividend yield with yield calculated for each of the past five years. [optional]  # noqa: E501
            five_year_dividend_growth_rate (float, none_type): The dividend growth rate is the annualized percentage rate of growth that a particular stock's dividend undergoes over five years of time. [optional]  # noqa: E501
            floating_shares_outstanding (float, none_type): Represents the number of shares outstanding less closely held shares for the period and date(s) requested. [optional]  # noqa: E501
            gross_margin (float, none_type): Gross profit margin is the proportion of money left over from revenues after accounting for the cost of goods sold, calculated by dividing gross profit by revenues.. [optional]  # noqa: E501
            inventory_turnover (float, none_type): Inventory turnover is a ratio showing how many times a company has sold and replaced inventory during a given period. [optional]  # noqa: E501
            long_term_debt_to_equity (float, none_type): Long-term debt consists of loans and financial obligations lasting over one year. The Debt/Equity (D/E) Ratio is calculated by dividing a company's total liabilities lasting over one year by its shareholder equity. [optional]  # noqa: E501
            net_income (float, none_type): This equals to net earnings (profit) calculated as sales less cost of goods sold, selling, general and administrative expenses, operating expenses, depreciation, interest, taxes and other expenses. [optional]  # noqa: E501
            number_of_employees (int, none_type): Represents the number of employees under the company's payroll as reported by the management to the shareholders within 90 days of the fiscal year-end.. [optional]  # noqa: E501
            payout_ratio (float, none_type): The dividend payout ratio is the ratio of the total amount of dividends paid out to shareholders relative to the net income of the company. [optional]  # noqa: E501
            pretax_margin (float, none_type): The pretax margin is the ratio of a company's pre-tax earnings to its total sales. [optional]  # noqa: E501
            price_to_book_ratio (float, none_type): Companies use the price-to-book ratio to compare a firm's market to book value by dividing price per share by book value per share (BVPS) .. [optional]  # noqa: E501
            price_to_cash_flow_ratio (float, none_type): The price-to-cash flow ratio is a stock valuation indicator or multiple that measures the value of a stock's price relative to its operating cash flow per share.. [optional]  # noqa: E501
            price_to_earnings_ratio (float, none_type): The price-to-earnings ratio (P/E ratio) is the ratio for valuing a company that measures its current share price relative to its per-share earnings. [optional]  # noqa: E501
            price_to_sales_ratio (float, none_type): The price-to-sales ratio (P/S ratio) is a valuation ratio that compares a company's stock price to its revenues. [optional]  # noqa: E501
            quick_ratio (float, none_type): The quick ratio measures the dollar amount of liquid assets available with the company against the dollar amount of its current liabilities. [optional]  # noqa: E501
            return_on_assets (float, none_type): Return on net assets (RONA) is a measure of financial performance calculated as net income divided by the sum of fixed assets and net working capital. [optional]  # noqa: E501
            return_on_equity (float, none_type): Return on equity (ROE) is a measure of financial performance calculated as net income divided by shareholders' equity. [optional]  # noqa: E501
            return_on_invested_capital (float, none_type): Return on Investment (ROI) measures how well an investment is performing. [optional]  # noqa: E501
            revenue_per_share (float, none_type): The portion of a company's revenue that is allocated to each share of common stock.. [optional]  # noqa: E501
            sales_per_employee (float, none_type): Revenue per employee is a ratio that is calculated as a company's total revenue divided by its current number of employees. [optional]  # noqa: E501
            sales_per_share (float, none_type): Sales per share is a ratio that computes the total revenue earned per share over a designated period. [optional]  # noqa: E501
            share_holder_equity (float, none_type): Shareholder equity represents the amount of financing the company experiences through common and preferred shares. [optional]  # noqa: E501
            three_year_average_yield (float, none_type): Average of the dividend yield with yield calculated for each of the past three years. [optional]  # noqa: E501
            three_year_dividend_growth_rate (float, none_type): The dividend growth rate is the annualized percentage rate of growth that a particular stock's dividend undergoes over three years of time. [optional]  # noqa: E501
            total_assets (float, none_type): Total amount of assets owned by entity.. [optional]  # noqa: E501
            total_debt_to_equity (float, none_type): The Debt/Equity (D/E) Ratio is calculated by dividing a company's total liabilities by its shareholder equity. [optional]  # noqa: E501
            total_revenue (float, none_type): Revenue is the amount of money (in Million) that a company actually receives during a specific period, including discounts and deductions for returned merchandise.. [optional]  # noqa: E501
            trailing_twelve_month_earnings_per_share (float, none_type): Earnings per share over the last 12 months.. [optional]  # noqa: E501
        """

        _check_type = kwargs.pop('_check_type', True)
        _spec_property_naming = kwargs.pop('_spec_property_naming', False)
        _path_to_item = kwargs.pop('_path_to_item', ())
        _configuration = kwargs.pop('_configuration', None)
        _visited_composed_classes = kwargs.pop('_visited_composed_classes', ())

        self = super(OpenApiModel, cls).__new__(cls)

        if args:
            raise ApiTypeError(
                "Invalid positional arguments=%s passed to %s. Remove those invalid positional arguments." % (
                    args,
                    self.__class__.__name__,
                ),
                path_to_item=_path_to_item,
                valid_classes=(self.__class__,),
            )

        self._data_store = {}
        self._check_type = _check_type
        self._spec_property_naming = _spec_property_naming
        self._path_to_item = _path_to_item
        self._configuration = _configuration
        self._visited_composed_classes = _visited_composed_classes + (self.__class__,)

        for var_name, var_value in kwargs.items():
            if var_name not in self.attribute_map and \
                        self._configuration is not None and \
                        self._configuration.discard_unknown_keys and \
                        self.additional_properties_type is None:
                # discard variable.
                continue
            setattr(self, var_name, var_value)
        return self

    required_properties = set([
        '_data_store',
        '_check_type',
        '_spec_property_naming',
        '_path_to_item',
        '_configuration',
        '_visited_composed_classes',
    ])

    @convert_js_args_to_python_args
    def __init__(self, *args, **kwargs):  # noqa: E501
        """Fundamentals - a model defined in OpenAPI

        Keyword Args:
            _check_type (bool): if True, values for parameters in openapi_types
                                will be type checked and a TypeError will be
                                raised if the wrong type is input.
                                Defaults to True
            _path_to_item (tuple/list): This is a list of keys or values to
                                drill down to the model in received_data
                                when deserializing a response
            _spec_property_naming (bool): True if the variable names in the input data
                                are serialized names, as specified in the OpenAPI document.
                                False if the variable names in the input data
                                are pythonic names, e.g. snake case (default)
            _configuration (Configuration): the instance to use when
                                deserializing a file_type parameter.
                                If passed, type conversion is attempted
                                If omitted no type conversion is done.
            _visited_composed_classes (tuple): This stores a tuple of
                                classes that we have traveled through so that
                                if we see that class again we will not use its
                                discriminator again.
                                When traveling through a discriminator, the
                                composed schema that is
                                is traveled through is added to this set.
                                For example if Animal has a discriminator
                                petType and we pass in "Dog", and the class Dog
                                allOf includes Animal, we move through Animal
                                once using the discriminator, and pick Dog.
                                Then in Dog, we will make an instance of the
                                Animal class but this time we won't travel
                                through its discriminator because we passed in
                                _visited_composed_classes = (Animal,)
            request_id (str): Identifier that was used for the request.. [optional]  # noqa: E501
            error (CompanyReportErrorObject): [optional]  # noqa: E501
            fsym_id (str, none_type): FactSet Regional Security Identifier. Six alpha-numeric characters, excluding vowels, with an -R suffix (XXXXXX-R). Identifies the security's best regional security data series per currency. For equities, all primary listings per region and currency are allocated a regional-level permanent identifier. The regional-level permanent identifier will be available once a SEDOL representing the region/currency has been allocated and the identifiers are on FactSet.. [optional]  # noqa: E501
            as_of_date (date, none_type): Date on which the specified fundamentals data or information is accurate or relevant.. [optional]  # noqa: E501
            currency (str, none_type): Currency code for the data. For a list of currency ISO codes, visit [Online Assistant Page #1470](https://oa.apps.factset.com/pages/1470).. [optional]  # noqa: E501
            asset_turnover_ratio (float, none_type): The asset turnover ratio measures the value of a company's sales or revenues relative to the value of its assets. [optional]  # noqa: E501
            book_value_per_share (float, none_type): Book value per common share is a formula used to calculate the per share value of a company based on common shareholders' equity in the company. [optional]  # noqa: E501
            cash_flow_per_share (float, none_type): Cash flow per share is calculated as a ratio, indicating the amount of cash a business generates based on a company's net income with the costs of depreciation and amortization added back. [optional]  # noqa: E501
            cash_per_share (float, none_type): Cash Per Share of Security. [optional]  # noqa: E501
            current_ratio (float, none_type): The current ratio is a liquidity ratio that measures a company's ability to pay short-term and long-term obligations. The ratio is calculated by comparing current assets to current liabilities. [optional]  # noqa: E501
            dividend (Dividend): [optional]  # noqa: E501
            earnings_per_share (float, none_type): Earnings per share (EPS) is the portion of a company's profit allocated to each share of common stock. [optional]  # noqa: E501
            ebit_margin (float, none_type): EBIT margin is an assessment of a firm's operating profitability as a percentage of its total revenue. It is equal to earnings before interest and tax (EBIT) divided by total revenue. [optional]  # noqa: E501
            ebitda_margin (float, none_type): EBITDA margin is an assessment of a firm's operating profitability as a percentage of its total revenue. It is equal to earnings before interest, tax, depreciation and amortization (EBITDA) divided by total revenue. [optional]  # noqa: E501
            enterprise_value (float, none_type): Enterprise Value (EV) is the measure of a company's total value for the period and date(s) requested in local currency by default. [optional]  # noqa: E501
            five_year_average_yield (float, none_type): Average of the dividend yield with yield calculated for each of the past five years. [optional]  # noqa: E501
            five_year_dividend_growth_rate (float, none_type): The dividend growth rate is the annualized percentage rate of growth that a particular stock's dividend undergoes over five years of time. [optional]  # noqa: E501
            floating_shares_outstanding (float, none_type): Represents the number of shares outstanding less closely held shares for the period and date(s) requested. [optional]  # noqa: E501
            gross_margin (float, none_type): Gross profit margin is the proportion of money left over from revenues after accounting for the cost of goods sold, calculated by dividing gross profit by revenues.. [optional]  # noqa: E501
            inventory_turnover (float, none_type): Inventory turnover is a ratio showing how many times a company has sold and replaced inventory during a given period. [optional]  # noqa: E501
            long_term_debt_to_equity (float, none_type): Long-term debt consists of loans and financial obligations lasting over one year. The Debt/Equity (D/E) Ratio is calculated by dividing a company's total liabilities lasting over one year by its shareholder equity. [optional]  # noqa: E501
            net_income (float, none_type): This equals to net earnings (profit) calculated as sales less cost of goods sold, selling, general and administrative expenses, operating expenses, depreciation, interest, taxes and other expenses. [optional]  # noqa: E501
            number_of_employees (int, none_type): Represents the number of employees under the company's payroll as reported by the management to the shareholders within 90 days of the fiscal year-end.. [optional]  # noqa: E501
            payout_ratio (float, none_type): The dividend payout ratio is the ratio of the total amount of dividends paid out to shareholders relative to the net income of the company. [optional]  # noqa: E501
            pretax_margin (float, none_type): The pretax margin is the ratio of a company's pre-tax earnings to its total sales. [optional]  # noqa: E501
            price_to_book_ratio (float, none_type): Companies use the price-to-book ratio to compare a firm's market to book value by dividing price per share by book value per share (BVPS) .. [optional]  # noqa: E501
            price_to_cash_flow_ratio (float, none_type): The price-to-cash flow ratio is a stock valuation indicator or multiple that measures the value of a stock's price relative to its operating cash flow per share.. [optional]  # noqa: E501
            price_to_earnings_ratio (float, none_type): The price-to-earnings ratio (P/E ratio) is the ratio for valuing a company that measures its current share price relative to its per-share earnings. [optional]  # noqa: E501
            price_to_sales_ratio (float, none_type): The price-to-sales ratio (P/S ratio) is a valuation ratio that compares a company's stock price to its revenues. [optional]  # noqa: E501
            quick_ratio (float, none_type): The quick ratio measures the dollar amount of liquid assets available with the company against the dollar amount of its current liabilities. [optional]  # noqa: E501
            return_on_assets (float, none_type): Return on net assets (RONA) is a measure of financial performance calculated as net income divided by the sum of fixed assets and net working capital. [optional]  # noqa: E501
            return_on_equity (float, none_type): Return on equity (ROE) is a measure of financial performance calculated as net income divided by shareholders' equity. [optional]  # noqa: E501
            return_on_invested_capital (float, none_type): Return on Investment (ROI) measures how well an investment is performing. [optional]  # noqa: E501
            revenue_per_share (float, none_type): The portion of a company's revenue that is allocated to each share of common stock.. [optional]  # noqa: E501
            sales_per_employee (float, none_type): Revenue per employee is a ratio that is calculated as a company's total revenue divided by its current number of employees. [optional]  # noqa: E501
            sales_per_share (float, none_type): Sales per share is a ratio that computes the total revenue earned per share over a designated period. [optional]  # noqa: E501
            share_holder_equity (float, none_type): Shareholder equity represents the amount of financing the company experiences through common and preferred shares. [optional]  # noqa: E501
            three_year_average_yield (float, none_type): Average of the dividend yield with yield calculated for each of the past three years. [optional]  # noqa: E501
            three_year_dividend_growth_rate (float, none_type): The dividend growth rate is the annualized percentage rate of growth that a particular stock's dividend undergoes over three years of time. [optional]  # noqa: E501
            total_assets (float, none_type): Total amount of assets owned by entity.. [optional]  # noqa: E501
            total_debt_to_equity (float, none_type): The Debt/Equity (D/E) Ratio is calculated by dividing a company's total liabilities by its shareholder equity. [optional]  # noqa: E501
            total_revenue (float, none_type): Revenue is the amount of money (in Million) that a company actually receives during a specific period, including discounts and deductions for returned merchandise.. [optional]  # noqa: E501
            trailing_twelve_month_earnings_per_share (float, none_type): Earnings per share over the last 12 months.. [optional]  # noqa: E501
        """

        _check_type = kwargs.pop('_check_type', True)
        _spec_property_naming = kwargs.pop('_spec_property_naming', False)
        _path_to_item = kwargs.pop('_path_to_item', ())
        _configuration = kwargs.pop('_configuration', None)
        _visited_composed_classes = kwargs.pop('_visited_composed_classes', ())

        if args:
            raise ApiTypeError(
                "Invalid positional arguments=%s passed to %s. Remove those invalid positional arguments." % (
                    args,
                    self.__class__.__name__,
                ),
                path_to_item=_path_to_item,
                valid_classes=(self.__class__,),
            )

        self._data_store = {}
        self._check_type = _check_type
        self._spec_property_naming = _spec_property_naming
        self._path_to_item = _path_to_item
        self._configuration = _configuration
        self._visited_composed_classes = _visited_composed_classes + (self.__class__,)

        for var_name, var_value in kwargs.items():
            if var_name not in self.attribute_map and \
                        self._configuration is not None and \
                        self._configuration.discard_unknown_keys and \
                        self.additional_properties_type is None:
                # discard variable.
                continue
            setattr(self, var_name, var_value)
            if var_name in self.read_only_vars:
                raise ApiAttributeError(f"`{var_name}` is a read-only attribute. Use `from_openapi_data` to instantiate "
                                     f"class with read only attributes.")




# flake8: noqa

# Import all APIs into this package.
# If you have many APIs here with many many models used in each API this may
# raise a `RecursionError`.
# In order to avoid this, import only the API that you directly need like:
#
#   from .api.batch_processing_api import BatchProcessingApi
#
# or import this package, but before doing it, use:
#
#   import sys
#   sys.setrecursionlimit(n)

# Import APIs into API package:
from fds.sdk.FactSetFundamentals.api.batch_processing_api import BatchProcessingApi
from fds.sdk.FactSetFundamentals.api.company_reports_api import CompanyReportsApi
from fds.sdk.FactSetFundamentals.api.fact_set_fundamentals_api import FactSetFundamentalsApi
from fds.sdk.FactSetFundamentals.api.metrics_api import MetricsApi
from fds.sdk.FactSetFundamentals.api.segments_api import SegmentsApi



"""
    FactSet Fundamentals API

    Gain access to current, comprehensive, and comparative information on securities in worldwide developed and emerging markets. Composed of annual and interim/quarterly data, detailed historical financial statement content, per-share data, and calculated ratios, FactSet Fundamentals provides you with the information you need for a global investment perspective.<p>This API is rate-limited to 10 requests per second and 10 concurrent requests per user.</p>   # noqa: E501

    The version of the OpenAPI document: 2.2.0
    Contact: api@factset.com
    Generated by: https://openapi-generator.tech
"""


from setuptools import setup, find_packages  # noqa: H301
import os

def read(filename):
  return open(os.path.join(os.path.dirname(__file__), filename)).read()

NAME = "fds.sdk.FactSetFundamentals"
VERSION = "2.3.2"
# To install the library, run the following
#
# python setup.py install
#
# prerequisite: setuptools
# http://pypi.python.org/pypi/setuptools

REQUIRES = [
  "urllib3 >= 1.25.3",
  "python-dateutil",
  "fds.sdk.utils >= 1.0.0",
]

setup(
    name=NAME,
    version=VERSION,
    description="FactSet Fundamentals client library for Python",
    author="FactSet Research Systems",
    url="https://github.com/FactSet/enterprise-sdk/tree/main/code/python/FactSetFundamentals/v2",
    keywords=["FactSet", "API", "SDK"],
    python_requires=">=3.7",
    install_requires=REQUIRES,
    packages=find_packages(exclude=["test", "tests"]),
    include_package_data=True,
    license="Apache License, Version 2.0",
    long_description_content_type="text/markdown",
    long_description=read("README.md")
)



"""
    FactSet Fundamentals API

    Gain access to current, comprehensive, and comparative information on securities in worldwide developed and emerging markets. Composed of annual and interim/quarterly data, detailed historical financial statement content, per-share data, and calculated ratios, FactSet Fundamentals provides you with the information you need for a global investment perspective.<p>This API is rate-limited to 10 requests per second and 10 concurrent requests per user.</p>   # noqa: E501

    The version of the OpenAPI document: 2.2.0
    Contact: api@factset.com
    Generated by: https://openapi-generator.tech
"""


import re  # noqa: F401
import sys  # noqa: F401
from multiprocessing.pool import ApplyResult
import typing

from fds.sdk.FactSetFundamentals.api_client import ApiClient, Endpoint as _Endpoint
from fds.sdk.FactSetFundamentals.model_utils import (  # noqa: F401
    check_allowed_values,
    check_validations,
    date,
    datetime,
    file_type,
    none_type,
    validate_and_convert_types
)
from fds.sdk.FactSetFundamentals.exceptions import ApiException
from fds.sdk.FactSetFundamentals.model.company_fundamentals_response import CompanyFundamentalsResponse
from fds.sdk.FactSetFundamentals.model.error_response import ErrorResponse
from fds.sdk.FactSetFundamentals.model.financial_response import FinancialResponse
from fds.sdk.FactSetFundamentals.model.profile_response import ProfileResponse





class CompanyReportsApi(object):
    """NOTE: This class is auto generated by OpenAPI Generator
    Ref: https://openapi-generator.tech

    Do not edit the class manually.
    """

    def __init__(self, api_client=None):
        if api_client is None:
            api_client = ApiClient()
        self.api_client = api_client

        self.get_fds_profiles_endpoint = _Endpoint(
            settings={
                'response_type': (
                  { 200: (ProfileResponse,), 400: (ErrorResponse,), 401: (ErrorResponse,), 403: (ErrorResponse,), 500: (ErrorResponse,),  },
                  None
                ),
                'auth': [
                    'FactSetApiKey',
                    'FactSetOAuth2'
                ],
                'endpoint_path': '/company-reports/profile',
                'operation_id': 'get_fds_profiles',
                'http_method': 'GET',
                'servers': None,
            },
            params_map={
                'all': [
                    'ids',
                ],
                'required': [
                    'ids',
                ],
                'nullable': [
                ],
                'enum': [
                ],
                'validation': [
                    'ids',
                ]
            },
            root_map={
                'validations': {
                    ('ids',): {

                        'max_items': 50,
                        'min_items': 1,
                    },
                },
                'allowed_values': {
                },
                'openapi_types': {
                    'ids':
                        ([str],),
                },
                'attribute_map': {
                    'ids': 'ids',
                },
                'location_map': {
                    'ids': 'query',
                },
                'collection_format_map': {
                    'ids': 'csv',
                }
            },
            headers_map={
                'accept': [
                    'application/json'
                ],
                'content_type': [],
            },
            api_client=api_client
        )

        self.get_fundamentals_endpoint = _Endpoint(
            settings={
                'response_type': (
                  { 200: (CompanyFundamentalsResponse,), 400: (ErrorResponse,), 401: (ErrorResponse,), 403: (ErrorResponse,), 500: (ErrorResponse,),  },
                  None
                ),
                'auth': [
                    'FactSetApiKey',
                    'FactSetOAuth2'
                ],
                'endpoint_path': '/company-reports/fundamentals',
                'operation_id': 'get_fundamentals',
                'http_method': 'GET',
                'servers': None,
            },
            params_map={
                'all': [
                    'ids',
                ],
                'required': [
                    'ids',
                ],
                'nullable': [
                ],
                'enum': [
                ],
                'validation': [
                    'ids',
                ]
            },
            root_map={
                'validations': {
                    ('ids',): {

                        'max_items': 50,
                        'min_items': 1,
                    },
                },
                'allowed_values': {
                },
                'openapi_types': {
                    'ids':
                        ([str],),
                },
                'attribute_map': {
                    'ids': 'ids',
                },
                'location_map': {
                    'ids': 'query',
                },
                'collection_format_map': {
                    'ids': 'csv',
                }
            },
            headers_map={
                'accept': [
                    'application/json'
                ],
                'content_type': [],
            },
            api_client=api_client
        )


    @staticmethod
    def apply_kwargs_defaults(kwargs, return_http_data_only, async_req):
        kwargs["async_req"] = async_req
        kwargs["_return_http_data_only"] = return_http_data_only
        kwargs["_preload_content"] = kwargs.get("_preload_content", True)
        kwargs["_request_timeout"] = kwargs.get("_request_timeout", None)
        kwargs["_check_input_type"] = kwargs.get("_check_input_type", True)
        kwargs["_check_return_type"] = kwargs.get("_check_return_type", True)
        kwargs["_spec_property_naming"] = kwargs.get("_spec_property_naming", False)
        kwargs["_content_type"] = kwargs.get("_content_type")
        kwargs["_host_index"] = kwargs.get("_host_index")


    def get_fds_profiles(
        self,
        ids,
        **kwargs
    ) -> ProfileResponse:
        """Returns company profile information for a specified list of identifiers.  # noqa: E501

        Retrieves a comprehensive overview of key profile details and information of specified list of identifiers. The response includes the company name, address, industry, sector, number of employees, CEO, business summary, exchange, market capitalization, shares outstanding, P/E ratio, year founded, and other details. All values provided in the response are absolute.   # noqa: E501
        This method makes a synchronous HTTP request. Returns the http data only

        Args:
            ids ([str]): The requested list of security identifiers. Accepted ID types include Market Tickers, SEDOL, ISINs, CUSIPs, or FactSet Permanent Ids. <p>***ids limit** =  50 per request*</p> 

        Keyword Args:
            _preload_content (bool): if False, the urllib3.HTTPResponse object
                will be returned without reading/decoding response data.
                Default is True. NOTE: if this API returns a file, it is the responsibility
                of the caller to close the file stream.
            _request_timeout (int/float/tuple): timeout setting for this request. If
                one number provided, it will be total request timeout. It can also
                be a pair (tuple) of (connection, read) timeouts.
                Default is None.
            _check_input_type (bool): specifies if type checking
                should be done one the data sent to the server.
                Default is True.
            _check_return_type (bool): specifies if type checking
                should be done one the data received from the server.
                Default is True.
            _spec_property_naming (bool): True if the variable names in the input data
                are serialized names, as specified in the OpenAPI document.
                False if the variable names in the input data
                are pythonic names, e.g. snake case (default)
            _content_type (str/None): force body content-type.
                Default is None and content-type will be predicted by allowed
                content-types and body.
            _host_index (int/None): specifies the index of the server
                that we want to use.
                Default is read from the configuration.
        Returns:
            ProfileResponse
                Response Object
        """
        self.apply_kwargs_defaults(kwargs=kwargs, return_http_data_only=True, async_req=False)
        kwargs['ids'] = \
            ids
        return self.get_fds_profiles_endpoint.call_with_http_info(**kwargs)

    def get_fds_profiles_with_http_info(
        self,
        ids,
        **kwargs
    ) -> typing.Tuple[ProfileResponse, int, typing.MutableMapping]:
        """Returns company profile information for a specified list of identifiers.  # noqa: E501

        Retrieves a comprehensive overview of key profile details and information of specified list of identifiers. The response includes the company name, address, industry, sector, number of employees, CEO, business summary, exchange, market capitalization, shares outstanding, P/E ratio, year founded, and other details. All values provided in the response are absolute.   # noqa: E501
        This method makes a synchronous HTTP request. Returns http data, http status and headers

        Args:
            ids ([str]): The requested list of security identifiers. Accepted ID types include Market Tickers, SEDOL, ISINs, CUSIPs, or FactSet Permanent Ids. <p>***ids limit** =  50 per request*</p> 

        Keyword Args:
            _preload_content (bool): if False, the urllib3.HTTPResponse object
                will be returned without reading/decoding response data.
                Default is True. NOTE: if this API returns a file, it is the responsibility
                of the caller to close the file stream.
            _request_timeout (int/float/tuple): timeout setting for this request. If
                one number provided, it will be total request timeout. It can also
                be a pair (tuple) of (connection, read) timeouts.
                Default is None.
            _check_input_type (bool): specifies if type checking
                should be done one the data sent to the server.
                Default is True.
            _check_return_type (bool): specifies if type checking
                should be done one the data received from the server.
                Default is True.
            _spec_property_naming (bool): True if the variable names in the input data
                are serialized names, as specified in the OpenAPI document.
                False if the variable names in the input data
                are pythonic names, e.g. snake case (default)
            _content_type (str/None): force body content-type.
                Default is None and content-type will be predicted by allowed
                content-types and body.
            _host_index (int/None): specifies the index of the server
                that we want to use.
                Default is read from the configuration.
        Returns:
            ProfileResponse
                Response Object
            int
                Http Status Code
            dict
                Dictionary of the response headers
        """
        self.apply_kwargs_defaults(kwargs=kwargs, return_http_data_only=False, async_req=False)
        kwargs['ids'] = \
            ids
        return self.get_fds_profiles_endpoint.call_with_http_info(**kwargs)

    def get_fds_profiles_async(
        self,
        ids,
        **kwargs
    ) -> "ApplyResult[ProfileResponse]":
        """Returns company profile information for a specified list of identifiers.  # noqa: E501

        Retrieves a comprehensive overview of key profile details and information of specified list of identifiers. The response includes the company name, address, industry, sector, number of employees, CEO, business summary, exchange, market capitalization, shares outstanding, P/E ratio, year founded, and other details. All values provided in the response are absolute.   # noqa: E501
        This method makes a asynchronous HTTP request. Returns the http data, wrapped in ApplyResult

        Args:
            ids ([str]): The requested list of security identifiers. Accepted ID types include Market Tickers, SEDOL, ISINs, CUSIPs, or FactSet Permanent Ids. <p>***ids limit** =  50 per request*</p> 

        Keyword Args:
            _preload_content (bool): if False, the urllib3.HTTPResponse object
                will be returned without reading/decoding response data.
                Default is True. NOTE: if this API returns a file, it is the responsibility
                of the caller to close the file stream.
            _request_timeout (int/float/tuple): timeout setting for this request. If
                one number provided, it will be total request timeout. It can also
                be a pair (tuple) of (connection, read) timeouts.
                Default is None.
            _check_input_type (bool): specifies if type checking
                should be done one the data sent to the server.
                Default is True.
            _check_return_type (bool): specifies if type checking
                should be done one the data received from the server.
                Default is True.
            _spec_property_naming (bool): True if the variable names in the input data
                are serialized names, as specified in the OpenAPI document.
                False if the variable names in the input data
                are pythonic names, e.g. snake case (default)
            _content_type (str/None): force body content-type.
                Default is None and content-type will be predicted by allowed
                content-types and body.
            _host_index (int/None): specifies the index of the server
                that we want to use.
                Default is read from the configuration.
        Returns:
            ApplyResult[ProfileResponse]
        """
        self.apply_kwargs_defaults(kwargs=kwargs, return_http_data_only=True, async_req=True)
        kwargs['ids'] = \
            ids
        return self.get_fds_profiles_endpoint.call_with_http_info(**kwargs)

    def get_fds_profiles_with_http_info_async(
        self,
        ids,
        **kwargs
    ) -> "ApplyResult[typing.Tuple[ProfileResponse, int, typing.MutableMapping]]":
        """Returns company profile information for a specified list of identifiers.  # noqa: E501

        Retrieves a comprehensive overview of key profile details and information of specified list of identifiers. The response includes the company name, address, industry, sector, number of employees, CEO, business summary, exchange, market capitalization, shares outstanding, P/E ratio, year founded, and other details. All values provided in the response are absolute.   # noqa: E501
        This method makes a asynchronous HTTP request. Returns http data, http status and headers, wrapped in ApplyResult

        Args:
            ids ([str]): The requested list of security identifiers. Accepted ID types include Market Tickers, SEDOL, ISINs, CUSIPs, or FactSet Permanent Ids. <p>***ids limit** =  50 per request*</p> 

        Keyword Args:
            _preload_content (bool): if False, the urllib3.HTTPResponse object
                will be returned without reading/decoding response data.
                Default is True. NOTE: if this API returns a file, it is the responsibility
                of the caller to close the file stream.
            _request_timeout (int/float/tuple): timeout setting for this request. If
                one number provided, it will be total request timeout. It can also
                be a pair (tuple) of (connection, read) timeouts.
                Default is None.
            _check_input_type (bool): specifies if type checking
                should be done one the data sent to the server.
                Default is True.
            _check_return_type (bool): specifies if type checking
                should be done one the data received from the server.
                Default is True.
            _spec_property_naming (bool): True if the variable names in the input data
                are serialized names, as specified in the OpenAPI document.
                False if the variable names in the input data
                are pythonic names, e.g. snake case (default)
            _content_type (str/None): force body content-type.
                Default is None and content-type will be predicted by allowed
                content-types and body.
            _host_index (int/None): specifies the index of the server
                that we want to use.
                Default is read from the configuration.
        Returns:
            ApplyResult[(ProfileResponse, int, typing.Dict)]
        """
        self.apply_kwargs_defaults(kwargs=kwargs, return_http_data_only=False, async_req=True)
        kwargs['ids'] = \
            ids
        return self.get_fds_profiles_endpoint.call_with_http_info(**kwargs)


    def get_fundamentals(
        self,
        ids,
        **kwargs
    ) -> CompanyFundamentalsResponse:
        """Returns company fundamentals for a given list of identifiers.  # noqa: E501

        Returns detailed insights on specified publicly traded company's various key financial measures or fundamentals like cash per share, dividend, EPS, EBIT etc. All values provided in the response are absolute.   # noqa: E501
        This method makes a synchronous HTTP request. Returns the http data only

        Args:
            ids ([str]): The requested list of security identifiers. Accepted ID types include Market Tickers, SEDOL, ISINs, CUSIPs, or FactSet Permanent Ids. <p>***ids limit** =  50 per request*</p> 

        Keyword Args:
            _preload_content (bool): if False, the urllib3.HTTPResponse object
                will be returned without reading/decoding response data.
                Default is True. NOTE: if this API returns a file, it is the responsibility
                of the caller to close the file stream.
            _request_timeout (int/float/tuple): timeout setting for this request. If
                one number provided, it will be total request timeout. It can also
                be a pair (tuple) of (connection, read) timeouts.
                Default is None.
            _check_input_type (bool): specifies if type checking
                should be done one the data sent to the server.
                Default is True.
            _check_return_type (bool): specifies if type checking
                should be done one the data received from the server.
                Default is True.
            _spec_property_naming (bool): True if the variable names in the input data
                are serialized names, as specified in the OpenAPI document.
                False if the variable names in the input data
                are pythonic names, e.g. snake case (default)
            _content_type (str/None): force body content-type.
                Default is None and content-type will be predicted by allowed
                content-types and body.
            _host_index (int/None): specifies the index of the server
                that we want to use.
                Default is read from the configuration.
        Returns:
            CompanyFundamentalsResponse
                Response Object
        """
        self.apply_kwargs_defaults(kwargs=kwargs, return_http_data_only=True, async_req=False)
        kwargs['ids'] = \
            ids
        return self.get_fundamentals_endpoint.call_with_http_info(**kwargs)

    def get_fundamentals_with_http_info(
        self,
        ids,
        **kwargs
    ) -> typing.Tuple[CompanyFundamentalsResponse, int, typing.MutableMapping]:
        """Returns company fundamentals for a given list of identifiers.  # noqa: E501

        Returns detailed insights on specified publicly traded company's various key financial measures or fundamentals like cash per share, dividend, EPS, EBIT etc. All values provided in the response are absolute.   # noqa: E501
        This method makes a synchronous HTTP request. Returns http data, http status and headers

        Args:
            ids ([str]): The requested list of security identifiers. Accepted ID types include Market Tickers, SEDOL, ISINs, CUSIPs, or FactSet Permanent Ids. <p>***ids limit** =  50 per request*</p> 

        Keyword Args:
            _preload_content (bool): if False, the urllib3.HTTPResponse object
                will be returned without reading/decoding response data.
                Default is True. NOTE: if this API returns a file, it is the responsibility
                of the caller to close the file stream.
            _request_timeout (int/float/tuple): timeout setting for this request. If
                one number provided, it will be total request timeout. It can also
                be a pair (tuple) of (connection, read) timeouts.
                Default is None.
            _check_input_type (bool): specifies if type checking
                should be done one the data sent to the server.
                Default is True.
            _check_return_type (bool): specifies if type checking
                should be done one the data received from the server.
                Default is True.
            _spec_property_naming (bool): True if the variable names in the input data
                are serialized names, as specified in the OpenAPI document.
                False if the variable names in the input data
                are pythonic names, e.g. snake case (default)
            _content_type (str/None): force body content-type.
                Default is None and content-type will be predicted by allowed
                content-types and body.
            _host_index (int/None): specifies the index of the server
                that we want to use.
                Default is read from the configuration.
        Returns:
            CompanyFundamentalsResponse
                Response Object
            int
                Http Status Code
            dict
                Dictionary of the response headers
        """
        self.apply_kwargs_defaults(kwargs=kwargs, return_http_data_only=False, async_req=False)
        kwargs['ids'] = \
            ids
        return self.get_fundamentals_endpoint.call_with_http_info(**kwargs)

    def get_fundamentals_async(
        self,
        ids,
        **kwargs
    ) -> "ApplyResult[CompanyFundamentalsResponse]":
        """Returns company fundamentals for a given list of identifiers.  # noqa: E501

        Returns detailed insights on specified publicly traded company's various key financial measures or fundamentals like cash per share, dividend, EPS, EBIT etc. All values provided in the response are absolute.   # noqa: E501
        This method makes a asynchronous HTTP request. Returns the http data, wrapped in ApplyResult

        Args:
            ids ([str]): The requested list of security identifiers. Accepted ID types include Market Tickers, SEDOL, ISINs, CUSIPs, or FactSet Permanent Ids. <p>***ids limit** =  50 per request*</p> 

        Keyword Args:
            _preload_content (bool): if False, the urllib3.HTTPResponse object
                will be returned without reading/decoding response data.
                Default is True. NOTE: if this API returns a file, it is the responsibility
                of the caller to close the file stream.
            _request_timeout (int/float/tuple): timeout setting for this request. If
                one number provided, it will be total request timeout. It can also
                be a pair (tuple) of (connection, read) timeouts.
                Default is None.
            _check_input_type (bool): specifies if type checking
                should be done one the data sent to the server.
                Default is True.
            _check_return_type (bool): specifies if type checking
                should be done one the data received from the server.
                Default is True.
            _spec_property_naming (bool): True if the variable names in the input data
                are serialized names, as specified in the OpenAPI document.
                False if the variable names in the input data
                are pythonic names, e.g. snake case (default)
            _content_type (str/None): force body content-type.
                Default is None and content-type will be predicted by allowed
                content-types and body.
            _host_index (int/None): specifies the index of the server
                that we want to use.
                Default is read from the configuration.
        Returns:
            ApplyResult[CompanyFundamentalsResponse]
        """
        self.apply_kwargs_defaults(kwargs=kwargs, return_http_data_only=True, async_req=True)
        kwargs['ids'] = \
            ids
        return self.get_fundamentals_endpoint.call_with_http_info(**kwargs)

    def get_fundamentals_with_http_info_async(
        self,
        ids,
        **kwargs
    ) -> "ApplyResult[typing.Tuple[CompanyFundamentalsResponse, int, typing.MutableMapping]]":
        """Returns company fundamentals for a given list of identifiers.  # noqa: E501

        Returns detailed insights on specified publicly traded company's various key financial measures or fundamentals like cash per share, dividend, EPS, EBIT etc. All values provided in the response are absolute.   # noqa: E501
        This method makes a asynchronous HTTP request. Returns http data, http status and headers, wrapped in ApplyResult

        Args:
            ids ([str]): The requested list of security identifiers. Accepted ID types include Market Tickers, SEDOL, ISINs, CUSIPs, or FactSet Permanent Ids. <p>***ids limit** =  50 per request*</p> 

        Keyword Args:
            _preload_content (bool): if False, the urllib3.HTTPResponse object
                will be returned without reading/decoding response data.
                Default is True. NOTE: if this API returns a file, it is the responsibility
                of the caller to close the file stream.
            _request_timeout (int/float/tuple): timeout setting for this request. If
                one number provided, it will be total request timeout. It can also
                be a pair (tuple) of (connection, read) timeouts.
                Default is None.
            _check_input_type (bool): specifies if type checking
                should be done one the data sent to the server.
                Default is True.
            _check_return_type (bool): specifies if type checking
                should be done one the data received from the server.
                Default is True.
            _spec_property_naming (bool): True if the variable names in the input data
                are serialized names, as specified in the OpenAPI document.
                False if the variable names in the input data
                are pythonic names, e.g. snake case (default)
            _content_type (str/None): force body content-type.
                Default is None and content-type will be predicted by allowed
                content-types and body.
            _host_index (int/None): specifies the index of the server
                that we want to use.
                Default is read from the configuration.
        Returns:
            ApplyResult[(CompanyFundamentalsResponse, int, typing.Dict)]
        """
        self.apply_kwargs_defaults(kwargs=kwargs, return_http_data_only=False, async_req=True)
        kwargs['ids'] = \
            ids
        return self.get_fundamentals_endpoint.call_with_http_info(**kwargs)





"""
    FactSet Fundamentals API

    Gain access to current, comprehensive, and comparative information on securities in worldwide developed and emerging markets. Composed of annual and interim/quarterly data, detailed historical financial statement content, per-share data, and calculated ratios, FactSet Fundamentals provides you with the information you need for a global investment perspective.<p>This API is rate-limited to 10 requests per second and 10 concurrent requests per user.</p>   # noqa: E501

    The version of the OpenAPI document: 2.2.0
    Contact: api@factset.com
    Generated by: https://openapi-generator.tech
"""


import re  # noqa: F401
import sys  # noqa: F401
from multiprocessing.pool import ApplyResult
import typing

from fds.sdk.FactSetFundamentals.api_client import ApiClient, Endpoint as _Endpoint
from fds.sdk.FactSetFundamentals.model_utils import (  # noqa: F401
    check_allowed_values,
    check_validations,
    date,
    datetime,
    file_type,
    none_type,
    validate_and_convert_types
)
from fds.sdk.FactSetFundamentals.exceptions import ApiException
from fds.sdk.FactSetFundamentals.model.error_response import ErrorResponse
from fds.sdk.FactSetFundamentals.model.metrics_response import MetricsResponse





class MetricsApi(object):
    """NOTE: This class is auto generated by OpenAPI Generator
    Ref: https://openapi-generator.tech

    Do not edit the class manually.
    """

    def __init__(self, api_client=None):
        if api_client is None:
            api_client = ApiClient()
        self.api_client = api_client

        self.get_fds_fundamentals_metrics_endpoint = _Endpoint(
            settings={
                'response_type': (
                  { 200: (MetricsResponse,), 400: (ErrorResponse,), 401: (ErrorResponse,), 403: (ErrorResponse,), 415: (ErrorResponse,), 500: (ErrorResponse,),  },
                  None
                ),
                'auth': [
                    'FactSetApiKey',
                    'FactSetOAuth2'
                ],
                'endpoint_path': '/metrics',
                'operation_id': 'get_fds_fundamentals_metrics',
                'http_method': 'GET',
                'servers': None,
            },
            params_map={
                'all': [
                    'category',
                    'subcategory',
                    'metric_data_type',
                ],
                'required': [],
                'nullable': [
                ],
                'enum': [
                    'category',
                    'subcategory',
                ],
                'validation': [
                ]
            },
            root_map={
                'validations': {
                },
                'allowed_values': {
                    ('category',): {

                        "INCOME_STATEMENT": "INCOME_STATEMENT",
                        "BALANCE_SHEET": "BALANCE_SHEET",
                        "CASH_FLOW": "CASH_FLOW",
                        "RATIOS": "RATIOS",
                        "FINANCIAL_SERVICES": "FINANCIAL_SERVICES",
                        "INDUSTRY_METRICS": "INDUSTRY_METRICS",
                        "PENSION_AND_POSTRETIREMENT": "PENSION_AND_POSTRETIREMENT",
                        "MARKET_DATA": "MARKET_DATA",
                        "MISCELLANEOUS": "MISCELLANEOUS",
                        "DATES": "DATES"
                    },
                    ('subcategory',): {

                        "ASSETS": "ASSETS",
                        "BALANCE_SHEET": "BALANCE_SHEET",
                        "HEALTHCARE": "HEALTHCARE",
                        "LIABILITIES": "LIABILITIES",
                        "PER_SHARE": "PER_SHARE",
                        "SHAREHOLDERS_EQUITY": "SHAREHOLDERS_EQUITY",
                        "SUPPLEMENTAL": "SUPPLEMENTAL",
                        "CASH_FLOW": "CASH_FLOW",
                        "CHANGE_IN_CASH": "CHANGE_IN_CASH",
                        "FINANCING": "FINANCING",
                        "INVESTING": "INVESTING",
                        "OPERATING": "OPERATING",
                        "DATES": "DATES",
                        "INCOME_STATEMENT": "INCOME_STATEMENT",
                        "NON-OPERATING": "NON-OPERATING",
                        "RETAIL": "RETAIL",
                        "AIRLINES": "AIRLINES",
                        "BANK": "BANK",
                        "BANKING": "BANKING",
                        "HOTELS_AND_GAMING": "HOTELS_AND_GAMING",
                        "METALS_AND_MINING": "METALS_AND_MINING",
                        "OIL_AND_GAS": "OIL_AND_GAS",
                        "PHARMACEUTICAL": "PHARMACEUTICAL",
                        "REIT": "REIT",
                        "MARKET_DATA": "MARKET_DATA",
                        "CLASSIFICATION": "CLASSIFICATION",
                        "CORPORATE_DATA": "CORPORATE_DATA",
                        "FINANCIAL_RECORDS": "FINANCIAL_RECORDS",
                        "INDICATOR": "INDICATOR",
                        "EMPLOYEES_AND_MANAGEMENT": "EMPLOYEES_AND_MANAGEMENT",
                        "PENSION_AND_POSTRETIREMENT": "PENSION_AND_POSTRETIREMENT",
                        "FINANCIAL": "FINANCIAL",
                        "GROWTH_RATE": "GROWTH_RATE",
                        "LIQUIDITY": "LIQUIDITY",
                        "PROFITABILITY": "PROFITABILITY",
                        "SIZE": "SIZE",
                        "VALUATION": "VALUATION",
                        "OTHER": "OTHER",
                        "HOMEBUILDING": "HOMEBUILDING",
                        "NET_INCOME": "NET_INCOME",
                        "TELECOM": "TELECOM",
                        "UTILITY": "UTILITY",
                        "INSURANCE": "INSURANCE"
                    },
                },
                'openapi_types': {
                    'category':
                        (str,),
                    'subcategory':
                        (str,),
                    'metric_data_type':
                        (str,),
                },
                'attribute_map': {
                    'category': 'category',
                    'subcategory': 'subcategory',
                    'metric_data_type': 'metricDataType',
                },
                'location_map': {
                    'category': 'query',
                    'subcategory': 'query',
                    'metric_data_type': 'query',
                },
                'collection_format_map': {
                }
            },
            headers_map={
                'accept': [
                    'application/json'
                ],
                'content_type': [],
            },
            api_client=api_client
        )


    @staticmethod
    def apply_kwargs_defaults(kwargs, return_http_data_only, async_req):
        kwargs["async_req"] = async_req
        kwargs["_return_http_data_only"] = return_http_data_only
        kwargs["_preload_content"] = kwargs.get("_preload_content", True)
        kwargs["_request_timeout"] = kwargs.get("_request_timeout", None)
        kwargs["_check_input_type"] = kwargs.get("_check_input_type", True)
        kwargs["_check_return_type"] = kwargs.get("_check_return_type", True)
        kwargs["_spec_property_naming"] = kwargs.get("_spec_property_naming", False)
        kwargs["_content_type"] = kwargs.get("_content_type")
        kwargs["_host_index"] = kwargs.get("_host_index")


    def get_fds_fundamentals_metrics(
        self,
        **kwargs
    ) -> MetricsResponse:
        """Returns available FactSet Fundamental metrics and ratios.  # noqa: E501

        Returns list of available FF_* metrics that can be used in the `metrics` parameter of related endpoints. These are related to FactSet Fundamentals standardized data. As Reported will be available in future endpoints. Leave Category and Subcategory blank to request all available items. The Endpoint Data model is optimized for time-series data with periodicity. Some items in this list are non-time series. **For methodology definitions, reference the `OApageID` or `OAurl` response items to launch the available methodology page.**   # noqa: E501
        This method makes a synchronous HTTP request. Returns the http data only


        Keyword Args:
            category (str): Filters the list of FF_* metrics by major category -   * **INCOME_STATEMENT** = Income Statement line items, such as Sales, Gross Profit, Net Income.   * **BALANCE_SHEET** = Balance Sheet line items, such as Assets, Liabilities, and Shareholders Equity.   * **CASH_FLOW** = Cash Flow Statement line items, such as Financing activities, Operation, and Per Share.   * **RATIOS** = Pre-calculated Ratios, including Financial, Growth Rates, Profitability, Liquidity, Size, and Valuation.   * **FINANCIAL_SERVICES** = Financial Statement Items modified for Financial Services companies.   * **INDUSTRY_METRICS** = Industry Specific Line Items or Modifications. View subcategory for list of Industries.   * **PENSION_AND_POSTRETIREMENT** = Accumulated Pension Benefit Obligations and related data.   * **MARKET_DATA** = General Market Data, such as Shares Outstanding. *Note - /factset-prices/prices/ endpoints may be better suited for pricing-related market data.*   * **MISCELLANEOUS** = Corporation Data, Financial Records details, Indicators.   * **DATES** = Relevant Dates . [optional]
            subcategory (str): Sub-Category Filter for the Primary Category Requested. Choose a related sub-category for the Category requested-   * **INCOME_STATEMENT** - INCOME_STATEMENT, NON-OPERATING, PER_SHARE, SUPPLEMENTAL, OTHER   * **BALANCE_SHEET** - ASSETS, BALANCE_SHEET, HEALTHCARE, LIABILITIES, PER_SHARE, SHAREHOLDERS_EQUITY, SUPPLEMENTAL   * **CASH_FLOW** - CASH_FLOW, CHANGE_IN_CASH, FINANCING, INVESTING, OPERATING, PER_SHARE, SUPPLEMENTAL   * **RATIOS** - FINANCIAL, GROWTH_RATE, LIQUIDITY, PROFITABILITY, SIZE, VALUATION   * **FINANCIAL_SERVICES** - BALANCE_SHEET, INCOME_STATEMENT, SUPPLEMENTAL   * **INDUSTRY_METRICS** - AIRLINES, BANKING, HOTELS_AND_GAMING, METALS_AND_MINING, OIL_AND_GAS, PHARMACEUTICAL, REIT, RETAIL, BANK, INSURANCE, UTILITY   * **PENSION_AND_POSTRETIREMENT** - PENSION_AND_POSTRETIREMENT   * **MARKET_DATA** - MARKET_DATA   * **MISCELLANEOUS** - CLASSIFICATION, CORPORATE_DATA, FINANCIAL_RECORDS, INDICATOR, EMPLOYEES_AND_MANAGEMENT   * **DATES** - DATES . [optional]
            metric_data_type (str): Returns general data type of the metrics like integer, float or strings, when left blankmetrics with all data types would be returned . [optional]
            _preload_content (bool): if False, the urllib3.HTTPResponse object
                will be returned without reading/decoding response data.
                Default is True. NOTE: if this API returns a file, it is the responsibility
                of the caller to close the file stream.
            _request_timeout (int/float/tuple): timeout setting for this request. If
                one number provided, it will be total request timeout. It can also
                be a pair (tuple) of (connection, read) timeouts.
                Default is None.
            _check_input_type (bool): specifies if type checking
                should be done one the data sent to the server.
                Default is True.
            _check_return_type (bool): specifies if type checking
                should be done one the data received from the server.
                Default is True.
            _spec_property_naming (bool): True if the variable names in the input data
                are serialized names, as specified in the OpenAPI document.
                False if the variable names in the input data
                are pythonic names, e.g. snake case (default)
            _content_type (str/None): force body content-type.
                Default is None and content-type will be predicted by allowed
                content-types and body.
            _host_index (int/None): specifies the index of the server
                that we want to use.
                Default is read from the configuration.
        Returns:
            MetricsResponse
                Response Object
        """
        self.apply_kwargs_defaults(kwargs=kwargs, return_http_data_only=True, async_req=False)
        return self.get_fds_fundamentals_metrics_endpoint.call_with_http_info(**kwargs)

    def get_fds_fundamentals_metrics_with_http_info(
        self,
        **kwargs
    ) -> typing.Tuple[MetricsResponse, int, typing.MutableMapping]:
        """Returns available FactSet Fundamental metrics and ratios.  # noqa: E501

        Returns list of available FF_* metrics that can be used in the `metrics` parameter of related endpoints. These are related to FactSet Fundamentals standardized data. As Reported will be available in future endpoints. Leave Category and Subcategory blank to request all available items. The Endpoint Data model is optimized for time-series data with periodicity. Some items in this list are non-time series. **For methodology definitions, reference the `OApageID` or `OAurl` response items to launch the available methodology page.**   # noqa: E501
        This method makes a synchronous HTTP request. Returns http data, http status and headers


        Keyword Args:
            category (str): Filters the list of FF_* metrics by major category -   * **INCOME_STATEMENT** = Income Statement line items, such as Sales, Gross Profit, Net Income.   * **BALANCE_SHEET** = Balance Sheet line items, such as Assets, Liabilities, and Shareholders Equity.   * **CASH_FLOW** = Cash Flow Statement line items, such as Financing activities, Operation, and Per Share.   * **RATIOS** = Pre-calculated Ratios, including Financial, Growth Rates, Profitability, Liquidity, Size, and Valuation.   * **FINANCIAL_SERVICES** = Financial Statement Items modified for Financial Services companies.   * **INDUSTRY_METRICS** = Industry Specific Line Items or Modifications. View subcategory for list of Industries.   * **PENSION_AND_POSTRETIREMENT** = Accumulated Pension Benefit Obligations and related data.   * **MARKET_DATA** = General Market Data, such as Shares Outstanding. *Note - /factset-prices/prices/ endpoints may be better suited for pricing-related market data.*   * **MISCELLANEOUS** = Corporation Data, Financial Records details, Indicators.   * **DATES** = Relevant Dates . [optional]
            subcategory (str): Sub-Category Filter for the Primary Category Requested. Choose a related sub-category for the Category requested-   * **INCOME_STATEMENT** - INCOME_STATEMENT, NON-OPERATING, PER_SHARE, SUPPLEMENTAL, OTHER   * **BALANCE_SHEET** - ASSETS, BALANCE_SHEET, HEALTHCARE, LIABILITIES, PER_SHARE, SHAREHOLDERS_EQUITY, SUPPLEMENTAL   * **CASH_FLOW** - CASH_FLOW, CHANGE_IN_CASH, FINANCING, INVESTING, OPERATING, PER_SHARE, SUPPLEMENTAL   * **RATIOS** - FINANCIAL, GROWTH_RATE, LIQUIDITY, PROFITABILITY, SIZE, VALUATION   * **FINANCIAL_SERVICES** - BALANCE_SHEET, INCOME_STATEMENT, SUPPLEMENTAL   * **INDUSTRY_METRICS** - AIRLINES, BANKING, HOTELS_AND_GAMING, METALS_AND_MINING, OIL_AND_GAS, PHARMACEUTICAL, REIT, RETAIL, BANK, INSURANCE, UTILITY   * **PENSION_AND_POSTRETIREMENT** - PENSION_AND_POSTRETIREMENT   * **MARKET_DATA** - MARKET_DATA   * **MISCELLANEOUS** - CLASSIFICATION, CORPORATE_DATA, FINANCIAL_RECORDS, INDICATOR, EMPLOYEES_AND_MANAGEMENT   * **DATES** - DATES . [optional]
            metric_data_type (str): Returns general data type of the metrics like integer, float or strings, when left blankmetrics with all data types would be returned . [optional]
            _preload_content (bool): if False, the urllib3.HTTPResponse object
                will be returned without reading/decoding response data.
                Default is True. NOTE: if this API returns a file, it is the responsibility
                of the caller to close the file stream.
            _request_timeout (int/float/tuple): timeout setting for this request. If
                one number provided, it will be total request timeout. It can also
                be a pair (tuple) of (connection, read) timeouts.
                Default is None.
            _check_input_type (bool): specifies if type checking
                should be done one the data sent to the server.
                Default is True.
            _check_return_type (bool): specifies if type checking
                should be done one the data received from the server.
                Default is True.
            _spec_property_naming (bool): True if the variable names in the input data
                are serialized names, as specified in the OpenAPI document.
                False if the variable names in the input data
                are pythonic names, e.g. snake case (default)
            _content_type (str/None): force body content-type.
                Default is None and content-type will be predicted by allowed
                content-types and body.
            _host_index (int/None): specifies the index of the server
                that we want to use.
                Default is read from the configuration.
        Returns:
            MetricsResponse
                Response Object
            int
                Http Status Code
            dict
                Dictionary of the response headers
        """
        self.apply_kwargs_defaults(kwargs=kwargs, return_http_data_only=False, async_req=False)
        return self.get_fds_fundamentals_metrics_endpoint.call_with_http_info(**kwargs)

    def get_fds_fundamentals_metrics_async(
        self,
        **kwargs
    ) -> "ApplyResult[MetricsResponse]":
        """Returns available FactSet Fundamental metrics and ratios.  # noqa: E501

        Returns list of available FF_* metrics that can be used in the `metrics` parameter of related endpoints. These are related to FactSet Fundamentals standardized data. As Reported will be available in future endpoints. Leave Category and Subcategory blank to request all available items. The Endpoint Data model is optimized for time-series data with periodicity. Some items in this list are non-time series. **For methodology definitions, reference the `OApageID` or `OAurl` response items to launch the available methodology page.**   # noqa: E501
        This method makes a asynchronous HTTP request. Returns the http data, wrapped in ApplyResult


        Keyword Args:
            category (str): Filters the list of FF_* metrics by major category -   * **INCOME_STATEMENT** = Income Statement line items, such as Sales, Gross Profit, Net Income.   * **BALANCE_SHEET** = Balance Sheet line items, such as Assets, Liabilities, and Shareholders Equity.   * **CASH_FLOW** = Cash Flow Statement line items, such as Financing activities, Operation, and Per Share.   * **RATIOS** = Pre-calculated Ratios, including Financial, Growth Rates, Profitability, Liquidity, Size, and Valuation.   * **FINANCIAL_SERVICES** = Financial Statement Items modified for Financial Services companies.   * **INDUSTRY_METRICS** = Industry Specific Line Items or Modifications. View subcategory for list of Industries.   * **PENSION_AND_POSTRETIREMENT** = Accumulated Pension Benefit Obligations and related data.   * **MARKET_DATA** = General Market Data, such as Shares Outstanding. *Note - /factset-prices/prices/ endpoints may be better suited for pricing-related market data.*   * **MISCELLANEOUS** = Corporation Data, Financial Records details, Indicators.   * **DATES** = Relevant Dates . [optional]
            subcategory (str): Sub-Category Filter for the Primary Category Requested. Choose a related sub-category for the Category requested-   * **INCOME_STATEMENT** - INCOME_STATEMENT, NON-OPERATING, PER_SHARE, SUPPLEMENTAL, OTHER   * **BALANCE_SHEET** - ASSETS, BALANCE_SHEET, HEALTHCARE, LIABILITIES, PER_SHARE, SHAREHOLDERS_EQUITY, SUPPLEMENTAL   * **CASH_FLOW** - CASH_FLOW, CHANGE_IN_CASH, FINANCING, INVESTING, OPERATING, PER_SHARE, SUPPLEMENTAL   * **RATIOS** - FINANCIAL, GROWTH_RATE, LIQUIDITY, PROFITABILITY, SIZE, VALUATION   * **FINANCIAL_SERVICES** - BALANCE_SHEET, INCOME_STATEMENT, SUPPLEMENTAL   * **INDUSTRY_METRICS** - AIRLINES, BANKING, HOTELS_AND_GAMING, METALS_AND_MINING, OIL_AND_GAS, PHARMACEUTICAL, REIT, RETAIL, BANK, INSURANCE, UTILITY   * **PENSION_AND_POSTRETIREMENT** - PENSION_AND_POSTRETIREMENT   * **MARKET_DATA** - MARKET_DATA   * **MISCELLANEOUS** - CLASSIFICATION, CORPORATE_DATA, FINANCIAL_RECORDS, INDICATOR, EMPLOYEES_AND_MANAGEMENT   * **DATES** - DATES . [optional]
            metric_data_type (str): Returns general data type of the metrics like integer, float or strings, when left blankmetrics with all data types would be returned . [optional]
            _preload_content (bool): if False, the urllib3.HTTPResponse object
                will be returned without reading/decoding response data.
                Default is True. NOTE: if this API returns a file, it is the responsibility
                of the caller to close the file stream.
            _request_timeout (int/float/tuple): timeout setting for this request. If
                one number provided, it will be total request timeout. It can also
                be a pair (tuple) of (connection, read) timeouts.
                Default is None.
            _check_input_type (bool): specifies if type checking
                should be done one the data sent to the server.
                Default is True.
            _check_return_type (bool): specifies if type checking
                should be done one the data received from the server.
                Default is True.
            _spec_property_naming (bool): True if the variable names in the input data
                are serialized names, as specified in the OpenAPI document.
                False if the variable names in the input data
                are pythonic names, e.g. snake case (default)
            _content_type (str/None): force body content-type.
                Default is None and content-type will be predicted by allowed
                content-types and body.
            _host_index (int/None): specifies the index of the server
                that we want to use.
                Default is read from the configuration.
        Returns:
            ApplyResult[MetricsResponse]
        """
        self.apply_kwargs_defaults(kwargs=kwargs, return_http_data_only=True, async_req=True)
        return self.get_fds_fundamentals_metrics_endpoint.call_with_http_info(**kwargs)

    def get_fds_fundamentals_metrics_with_http_info_async(
        self,
        **kwargs
    ) -> "ApplyResult[typing.Tuple[MetricsResponse, int, typing.MutableMapping]]":
        """Returns available FactSet Fundamental metrics and ratios.  # noqa: E501

        Returns list of available FF_* metrics that can be used in the `metrics` parameter of related endpoints. These are related to FactSet Fundamentals standardized data. As Reported will be available in future endpoints. Leave Category and Subcategory blank to request all available items. The Endpoint Data model is optimized for time-series data with periodicity. Some items in this list are non-time series. **For methodology definitions, reference the `OApageID` or `OAurl` response items to launch the available methodology page.**   # noqa: E501
        This method makes a asynchronous HTTP request. Returns http data, http status and headers, wrapped in ApplyResult


        Keyword Args:
            category (str): Filters the list of FF_* metrics by major category -   * **INCOME_STATEMENT** = Income Statement line items, such as Sales, Gross Profit, Net Income.   * **BALANCE_SHEET** = Balance Sheet line items, such as Assets, Liabilities, and Shareholders Equity.   * **CASH_FLOW** = Cash Flow Statement line items, such as Financing activities, Operation, and Per Share.   * **RATIOS** = Pre-calculated Ratios, including Financial, Growth Rates, Profitability, Liquidity, Size, and Valuation.   * **FINANCIAL_SERVICES** = Financial Statement Items modified for Financial Services companies.   * **INDUSTRY_METRICS** = Industry Specific Line Items or Modifications. View subcategory for list of Industries.   * **PENSION_AND_POSTRETIREMENT** = Accumulated Pension Benefit Obligations and related data.   * **MARKET_DATA** = General Market Data, such as Shares Outstanding. *Note - /factset-prices/prices/ endpoints may be better suited for pricing-related market data.*   * **MISCELLANEOUS** = Corporation Data, Financial Records details, Indicators.   * **DATES** = Relevant Dates . [optional]
            subcategory (str): Sub-Category Filter for the Primary Category Requested. Choose a related sub-category for the Category requested-   * **INCOME_STATEMENT** - INCOME_STATEMENT, NON-OPERATING, PER_SHARE, SUPPLEMENTAL, OTHER   * **BALANCE_SHEET** - ASSETS, BALANCE_SHEET, HEALTHCARE, LIABILITIES, PER_SHARE, SHAREHOLDERS_EQUITY, SUPPLEMENTAL   * **CASH_FLOW** - CASH_FLOW, CHANGE_IN_CASH, FINANCING, INVESTING, OPERATING, PER_SHARE, SUPPLEMENTAL   * **RATIOS** - FINANCIAL, GROWTH_RATE, LIQUIDITY, PROFITABILITY, SIZE, VALUATION   * **FINANCIAL_SERVICES** - BALANCE_SHEET, INCOME_STATEMENT, SUPPLEMENTAL   * **INDUSTRY_METRICS** - AIRLINES, BANKING, HOTELS_AND_GAMING, METALS_AND_MINING, OIL_AND_GAS, PHARMACEUTICAL, REIT, RETAIL, BANK, INSURANCE, UTILITY   * **PENSION_AND_POSTRETIREMENT** - PENSION_AND_POSTRETIREMENT   * **MARKET_DATA** - MARKET_DATA   * **MISCELLANEOUS** - CLASSIFICATION, CORPORATE_DATA, FINANCIAL_RECORDS, INDICATOR, EMPLOYEES_AND_MANAGEMENT   * **DATES** - DATES . [optional]
            metric_data_type (str): Returns general data type of the metrics like integer, float or strings, when left blankmetrics with all data types would be returned . [optional]
            _preload_content (bool): if False, the urllib3.HTTPResponse object
                will be returned without reading/decoding response data.
                Default is True. NOTE: if this API returns a file, it is the responsibility
                of the caller to close the file stream.
            _request_timeout (int/float/tuple): timeout setting for this request. If
                one number provided, it will be total request timeout. It can also
                be a pair (tuple) of (connection, read) timeouts.
                Default is None.
            _check_input_type (bool): specifies if type checking
                should be done one the data sent to the server.
                Default is True.
            _check_return_type (bool): specifies if type checking
                should be done one the data received from the server.
                Default is True.
            _spec_property_naming (bool): True if the variable names in the input data
                are serialized names, as specified in the OpenAPI document.
                False if the variable names in the input data
                are pythonic names, e.g. snake case (default)
            _content_type (str/None): force body content-type.
                Default is None and content-type will be predicted by allowed
                content-types and body.
            _host_index (int/None): specifies the index of the server
                that we want to use.
                Default is read from the configuration.
        Returns:
            ApplyResult[(MetricsResponse, int, typing.Dict)]
        """
        self.apply_kwargs_defaults(kwargs=kwargs, return_http_data_only=False, async_req=True)
        return self.get_fds_fundamentals_metrics_endpoint.call_with_http_info(**kwargs)


